#define TRACEBACK write (gol,'("in ",a," (",a,", line",i5,")")') rname, __FILE__, __LINE__; call goErr
#define IF_NOTOK_RETURN(action) if (status/=0) then; TRACEBACK; action; return; end if
#define IF_ERROR_RETURN(action) if (status> 0) then; TRACEBACK; action; return; end if
#include "tm5.inc"

! Quick access to a single variable from an HDF or netCDF file, via MDF interface.

module ReadFromFile_MDF
  
  use GO,		only: gol, goErr, goPr, goLabel
  use MDF

  implicit none

  character(len=*), parameter	:: mname = 'ReadFromFile_MDF'

  public

  ! This interface is not complete.  If you need a differently-typed or -shaped
  ! variable read function, just add it.
  interface ReadFromFile
	module procedure ReadFromFile_MDF_c1_1d
	module procedure ReadFromFile_MDF_c1_2d
	module procedure ReadFromFile_MDF_c1_3d
	module procedure ReadFromFile_MDF_c1_4d
	module procedure ReadFromFile_MDF_c1_5d
	module procedure ReadFromFile_MDF_i4_1d
	module procedure ReadFromFile_MDF_i4_2d
	module procedure ReadFromFile_MDF_i4_3d
	module procedure ReadFromFile_MDF_i4_4d
	module procedure ReadFromFile_MDF_i4_5d
	module procedure ReadFromFile_MDF_i2_1d
	module procedure ReadFromFile_MDF_i2_2d
	module procedure ReadFromFile_MDF_i2_3d
	module procedure ReadFromFile_MDF_i2_4d
	module procedure ReadFromFile_MDF_i2_5d
	module procedure ReadFromFile_MDF_i8_1d
	module procedure ReadFromFile_MDF_i8_2d
	module procedure ReadFromFile_MDF_i8_3d
	module procedure ReadFromFile_MDF_i8_4d
	module procedure ReadFromFile_MDF_i8_5d
	module procedure ReadFromFile_MDF_r8_1d
	module procedure ReadFromFile_MDF_r8_2d
	module procedure ReadFromFile_MDF_r8_3d
	module procedure ReadFromFile_MDF_r8_4d
	module procedure ReadFromFile_MDF_r8_5d
	module procedure ReadFromFile_MDF_r4_1d
	module procedure ReadFromFile_MDF_r4_2d
	module procedure ReadFromFile_MDF_r4_3d
	module procedure ReadFromFile_MDF_r4_4d
	module procedure ReadFromFile_MDF_r4_5d
  end interface

contains

  subroutine ReadFromFile_MDF_c1_1d(fname,ftype,vname,buffer,lenient_typing,status)

    character(len=*), intent(in)	:: fname  ! file name
    integer, intent(in)			:: ftype  ! MDF_HDF4, MDF_NETCDF, MDF_NETCDF4, MDF_HDF5
    character(len=*), intent(in)	:: vname  ! variable name
    character(1), intent(out)		:: buffer(:)
    logical, optional, intent(in)	:: lenient_typing
    integer, intent(out)		:: status

    integer				:: hid
    integer				:: vid
    integer				:: xtype
    integer				:: ndims

    character(len=*), parameter		:: rname = mname//'/ReadFromFile_MDF_c1_1d'

    call MDF_open(fname,ftype,mode=MDF_READ,hid=hid,status=status)
    IF_NOTOK_RETURN(status=1)

    call MDF_Inq_VarID(hid,vname,vid,status)
    IF_NOTOK_RETURN(status=1)

    call MDF_Inquire_Variable(hid,vid,status,xtype=xtype,ndims=ndims)
    IF_NOTOK_RETURN(status=1)
    
    if(ndims /= 1) then
       write (gol,'("[ReadFromFile_MDF_c1_1d]  Variable """,a,""" in file """,a,""":")') trim(vname),trim(fname);call goPr
       write (gol,'("[ReadFromFile_MDF_c1_1d]    Was expecting 1 dimensions but found ",i,".")') ndims;call goPr
       write (gol,'("[ReadFromFile_MDF_c1_1d]    Aborting.")');call goErr
       status=1;return
    endif

    if(xtype /= MDF_CHAR) then
       write (gol,'("[ReadFromFile_MDF_c1_1d]  Variable """,a,""" in file """,a,""":")') trim(vname),trim(fname);call goPr
       write (gol,'("[ReadFromFile_MDF_c1_1d]    Was expecting """,a,""" type, but found """,a,""" type.")') trim(MDF_DATATYPE_NAME(MDF_CHAR)),trim(MDF_DATATYPE_NAME(xtype));call goPr
       if(.not. present(lenient_typing)) then
          write (gol,'("[ReadFromFile_MDF_c1_1d]    Aborting.")');call goErr
          status=1;return
       else 
          write (gol,'("[ReadFromFile_MDF_c1_1d]    Using default type promotion.")'); call goPr
       endif
    endif

    call MDF_Get_Var(hid,vid,buffer,status)
    IF_NOTOK_RETURN(status=1)

    call MDF_Close(hid,status)
    IF_NOTOK_RETURN(status=1)

    status = 0

  end subroutine ReadFromFile_MDF_c1_1d

  subroutine ReadFromFile_MDF_c1_2d(fname,ftype,vname,buffer,lenient_typing,status)

    character(len=*), intent(in)	:: fname  ! file name
    integer, intent(in)			:: ftype  ! MDF_HDF4, MDF_NETCDF, MDF_NETCDF4, MDF_HDF5
    character(len=*), intent(in)	:: vname  ! variable name
    character(1), intent(out)		:: buffer(:,:)
    logical, optional, intent(in)	:: lenient_typing
    integer, intent(out)		:: status

    integer				:: hid
    integer				:: vid
    integer				:: xtype
    integer				:: ndims

    character(len=*), parameter		:: rname = mname//'/ReadFromFile_MDF_c1_2d'

    call MDF_open(fname,ftype,mode=MDF_READ,hid=hid,status=status)
    IF_NOTOK_RETURN(status=1)

    call MDF_Inq_VarID(hid,vname,vid,status)
    IF_NOTOK_RETURN(status=1)

    call MDF_Inquire_Variable(hid,vid,status,xtype=xtype,ndims=ndims)
    IF_NOTOK_RETURN(status=1)
    
    if(ndims /= 2) then
       write (gol,'("[ReadFromFile_MDF_c1_2d]  Variable """,a,""" in file """,a,""":")') trim(vname),trim(fname);call goPr
       write (gol,'("[ReadFromFile_MDF_c1_2d]    Was expecting 2 dimensions but found ",i,".")') ndims;call goPr
       write (gol,'("[ReadFromFile_MDF_c1_2d]    Aborting.")');call goErr
       status=1;return
    endif

    if(xtype /= MDF_CHAR) then
       write (gol,'("[ReadFromFile_MDF_c1_2d]  Variable """,a,""" in file """,a,""":")') trim(vname),trim(fname);call goPr
       write (gol,'("[ReadFromFile_MDF_c1_2d]    Was expecting """,a,""" type, but found """,a,""" type.")') trim(MDF_DATATYPE_NAME(MDF_CHAR)),trim(MDF_DATATYPE_NAME(xtype));call goPr
       if(.not. present(lenient_typing)) then
          write (gol,'("[ReadFromFile_MDF_c1_2d]    Aborting.")');call goErr
          status=1;return
       else 
          write (gol,'("[ReadFromFile_MDF_c1_2d]    Using default type promotion.")'); call goPr
       endif
    endif

    call MDF_Get_Var(hid,vid,buffer,status)
    IF_NOTOK_RETURN(status=1)

    call MDF_Close(hid,status)
    IF_NOTOK_RETURN(status=1)

    status = 0

  end subroutine ReadFromFile_MDF_c1_2d

  subroutine ReadFromFile_MDF_c1_3d(fname,ftype,vname,buffer,lenient_typing,status)

    character(len=*), intent(in)	:: fname  ! file name
    integer, intent(in)			:: ftype  ! MDF_HDF4, MDF_NETCDF, MDF_NETCDF4, MDF_HDF5
    character(len=*), intent(in)	:: vname  ! variable name
    character(1), intent(out)		:: buffer(:,:,:)
    logical, optional, intent(in)	:: lenient_typing
    integer, intent(out)		:: status

    integer				:: hid
    integer				:: vid
    integer				:: xtype
    integer				:: ndims

    character(len=*), parameter		:: rname = mname//'/ReadFromFile_MDF_c1_3d'

    call MDF_open(fname,ftype,mode=MDF_READ,hid=hid,status=status)
    IF_NOTOK_RETURN(status=1)

    call MDF_Inq_VarID(hid,vname,vid,status)
    IF_NOTOK_RETURN(status=1)

    call MDF_Inquire_Variable(hid,vid,status,xtype=xtype,ndims=ndims)
    IF_NOTOK_RETURN(status=1)
    
    if(ndims /= 3) then
       write (gol,'("[ReadFromFile_MDF_c1_3d]  Variable """,a,""" in file """,a,""":")') trim(vname),trim(fname);call goPr
       write (gol,'("[ReadFromFile_MDF_c1_3d]    Was expecting 3 dimensions but found ",i,".")') ndims;call goPr
       write (gol,'("[ReadFromFile_MDF_c1_3d]    Aborting.")');call goErr
       status=1;return
    endif

    if(xtype /= MDF_CHAR) then
       write (gol,'("[ReadFromFile_MDF_c1_3d]  Variable """,a,""" in file """,a,""":")') trim(vname),trim(fname);call goPr
       write (gol,'("[ReadFromFile_MDF_c1_3d]    Was expecting """,a,""" type, but found """,a,""" type.")') trim(MDF_DATATYPE_NAME(MDF_CHAR)),trim(MDF_DATATYPE_NAME(xtype));call goPr
       if(.not. present(lenient_typing)) then
          write (gol,'("[ReadFromFile_MDF_c1_3d]    Aborting.")');call goErr
          status=1;return
       else 
          write (gol,'("[ReadFromFile_MDF_c1_3d]    Using default type promotion.")'); call goPr
       endif
    endif

    call MDF_Get_Var(hid,vid,buffer,status)
    IF_NOTOK_RETURN(status=1)

    call MDF_Close(hid,status)
    IF_NOTOK_RETURN(status=1)

    status = 0

  end subroutine ReadFromFile_MDF_c1_3d

  subroutine ReadFromFile_MDF_c1_4d(fname,ftype,vname,buffer,lenient_typing,status)

    character(len=*), intent(in)	:: fname  ! file name
    integer, intent(in)			:: ftype  ! MDF_HDF4, MDF_NETCDF, MDF_NETCDF4, MDF_HDF5
    character(len=*), intent(in)	:: vname  ! variable name
    character(1), intent(out)		:: buffer(:,:,:,:)
    logical, optional, intent(in)	:: lenient_typing
    integer, intent(out)		:: status

    integer				:: hid
    integer				:: vid
    integer				:: xtype
    integer				:: ndims

    character(len=*), parameter		:: rname = mname//'/ReadFromFile_MDF_c1_4d'

    call MDF_open(fname,ftype,mode=MDF_READ,hid=hid,status=status)
    IF_NOTOK_RETURN(status=1)

    call MDF_Inq_VarID(hid,vname,vid,status)
    IF_NOTOK_RETURN(status=1)

    call MDF_Inquire_Variable(hid,vid,status,xtype=xtype,ndims=ndims)
    IF_NOTOK_RETURN(status=1)
    
    if(ndims /= 4) then
       write (gol,'("[ReadFromFile_MDF_c1_4d]  Variable """,a,""" in file """,a,""":")') trim(vname),trim(fname);call goPr
       write (gol,'("[ReadFromFile_MDF_c1_4d]    Was expecting 4 dimensions but found ",i,".")') ndims;call goPr
       write (gol,'("[ReadFromFile_MDF_c1_4d]    Aborting.")');call goErr
       status=1;return
    endif

    if(xtype /= MDF_CHAR) then
       write (gol,'("[ReadFromFile_MDF_c1_4d]  Variable """,a,""" in file """,a,""":")') trim(vname),trim(fname);call goPr
       write (gol,'("[ReadFromFile_MDF_c1_4d]    Was expecting """,a,""" type, but found """,a,""" type.")') trim(MDF_DATATYPE_NAME(MDF_CHAR)),trim(MDF_DATATYPE_NAME(xtype));call goPr
       if(.not. present(lenient_typing)) then
          write (gol,'("[ReadFromFile_MDF_c1_4d]    Aborting.")');call goErr
          status=1;return
       else 
          write (gol,'("[ReadFromFile_MDF_c1_4d]    Using default type promotion.")'); call goPr
       endif
    endif

    call MDF_Get_Var(hid,vid,buffer,status)
    IF_NOTOK_RETURN(status=1)

    call MDF_Close(hid,status)
    IF_NOTOK_RETURN(status=1)

    status = 0

  end subroutine ReadFromFile_MDF_c1_4d

  subroutine ReadFromFile_MDF_c1_5d(fname,ftype,vname,buffer,lenient_typing,status)

    character(len=*), intent(in)	:: fname  ! file name
    integer, intent(in)			:: ftype  ! MDF_HDF4, MDF_NETCDF, MDF_NETCDF4, MDF_HDF5
    character(len=*), intent(in)	:: vname  ! variable name
    character(1), intent(out)		:: buffer(:,:,:,:,:)
    logical, optional, intent(in)	:: lenient_typing
    integer, intent(out)		:: status

    integer				:: hid
    integer				:: vid
    integer				:: xtype
    integer				:: ndims

    character(len=*), parameter		:: rname = mname//'/ReadFromFile_MDF_c1_5d'

    call MDF_open(fname,ftype,mode=MDF_READ,hid=hid,status=status)
    IF_NOTOK_RETURN(status=1)

    call MDF_Inq_VarID(hid,vname,vid,status)
    IF_NOTOK_RETURN(status=1)

    call MDF_Inquire_Variable(hid,vid,status,xtype=xtype,ndims=ndims)
    IF_NOTOK_RETURN(status=1)
    
    if(ndims /= 5) then
       write (gol,'("[ReadFromFile_MDF_c1_5d]  Variable """,a,""" in file """,a,""":")') trim(vname),trim(fname);call goPr
       write (gol,'("[ReadFromFile_MDF_c1_5d]    Was expecting 5 dimensions but found ",i,".")') ndims;call goPr
       write (gol,'("[ReadFromFile_MDF_c1_5d]    Aborting.")');call goErr
       status=1;return
    endif

    if(xtype /= MDF_CHAR) then
       write (gol,'("[ReadFromFile_MDF_c1_5d]  Variable """,a,""" in file """,a,""":")') trim(vname),trim(fname);call goPr
       write (gol,'("[ReadFromFile_MDF_c1_5d]    Was expecting """,a,""" type, but found """,a,""" type.")') trim(MDF_DATATYPE_NAME(MDF_CHAR)),trim(MDF_DATATYPE_NAME(xtype));call goPr
       if(.not. present(lenient_typing)) then
          write (gol,'("[ReadFromFile_MDF_c1_5d]    Aborting.")');call goErr
          status=1;return
       else 
          write (gol,'("[ReadFromFile_MDF_c1_5d]    Using default type promotion.")'); call goPr
       endif
    endif

    call MDF_Get_Var(hid,vid,buffer,status)
    IF_NOTOK_RETURN(status=1)

    call MDF_Close(hid,status)
    IF_NOTOK_RETURN(status=1)

    status = 0

  end subroutine ReadFromFile_MDF_c1_5d

  subroutine ReadFromFile_MDF_i4_1d(fname,ftype,vname,buffer,lenient_typing,status)

    character(len=*), intent(in)	:: fname  ! file name
    integer, intent(in)			:: ftype  ! MDF_HDF4, MDF_NETCDF, MDF_NETCDF4, MDF_HDF5
    character(len=*), intent(in)	:: vname  ! variable name
    integer(4), intent(out)		:: buffer(:)
    logical, optional, intent(in)	:: lenient_typing
    integer, intent(out)		:: status

    integer				:: hid
    integer				:: vid
    integer				:: xtype
    integer				:: ndims

    character(len=*), parameter		:: rname = mname//'/ReadFromFile_MDF_i4_1d'

    call MDF_open(fname,ftype,mode=MDF_READ,hid=hid,status=status)
    IF_NOTOK_RETURN(status=1)

    call MDF_Inq_VarID(hid,vname,vid,status)
    IF_NOTOK_RETURN(status=1)

    call MDF_Inquire_Variable(hid,vid,status,xtype=xtype,ndims=ndims)
    IF_NOTOK_RETURN(status=1)
    
    if(ndims /= 1) then
       write (gol,'("[ReadFromFile_MDF_i4_1d]  Variable """,a,""" in file """,a,""":")') trim(vname),trim(fname);call goPr
       write (gol,'("[ReadFromFile_MDF_i4_1d]    Was expecting 1 dimensions but found ",i,".")') ndims;call goPr
       write (gol,'("[ReadFromFile_MDF_i4_1d]    Aborting.")');call goErr
       status=1;return
    endif

    if(xtype /= MDF_INT) then
       write (gol,'("[ReadFromFile_MDF_i4_1d]  Variable """,a,""" in file """,a,""":")') trim(vname),trim(fname);call goPr
       write (gol,'("[ReadFromFile_MDF_i4_1d]    Was expecting """,a,""" type, but found """,a,""" type.")') trim(MDF_DATATYPE_NAME(MDF_INT)),trim(MDF_DATATYPE_NAME(xtype));call goPr
       if(.not. present(lenient_typing)) then
          write (gol,'("[ReadFromFile_MDF_i4_1d]    Aborting.")');call goErr
          status=1;return
       else 
          write (gol,'("[ReadFromFile_MDF_i4_1d]    Using default type promotion.")'); call goPr
       endif
    endif

    call MDF_Get_Var(hid,vid,buffer,status)
    IF_NOTOK_RETURN(status=1)

    call MDF_Close(hid,status)
    IF_NOTOK_RETURN(status=1)

    status = 0

  end subroutine ReadFromFile_MDF_i4_1d

  subroutine ReadFromFile_MDF_i4_2d(fname,ftype,vname,buffer,lenient_typing,status)

    character(len=*), intent(in)	:: fname  ! file name
    integer, intent(in)			:: ftype  ! MDF_HDF4, MDF_NETCDF, MDF_NETCDF4, MDF_HDF5
    character(len=*), intent(in)	:: vname  ! variable name
    integer(4), intent(out)		:: buffer(:,:)
    logical, optional, intent(in)	:: lenient_typing
    integer, intent(out)		:: status

    integer				:: hid
    integer				:: vid
    integer				:: xtype
    integer				:: ndims

    character(len=*), parameter		:: rname = mname//'/ReadFromFile_MDF_i4_2d'

    call MDF_open(fname,ftype,mode=MDF_READ,hid=hid,status=status)
    IF_NOTOK_RETURN(status=1)

    call MDF_Inq_VarID(hid,vname,vid,status)
    IF_NOTOK_RETURN(status=1)

    call MDF_Inquire_Variable(hid,vid,status,xtype=xtype,ndims=ndims)
    IF_NOTOK_RETURN(status=1)
    
    if(ndims /= 2) then
       write (gol,'("[ReadFromFile_MDF_i4_2d]  Variable """,a,""" in file """,a,""":")') trim(vname),trim(fname);call goPr
       write (gol,'("[ReadFromFile_MDF_i4_2d]    Was expecting 2 dimensions but found ",i,".")') ndims;call goPr
       write (gol,'("[ReadFromFile_MDF_i4_2d]    Aborting.")');call goErr
       status=1;return
    endif

    if(xtype /= MDF_INT) then
       write (gol,'("[ReadFromFile_MDF_i4_2d]  Variable """,a,""" in file """,a,""":")') trim(vname),trim(fname);call goPr
       write (gol,'("[ReadFromFile_MDF_i4_2d]    Was expecting """,a,""" type, but found """,a,""" type.")') trim(MDF_DATATYPE_NAME(MDF_INT)),trim(MDF_DATATYPE_NAME(xtype));call goPr
       if(.not. present(lenient_typing)) then
          write (gol,'("[ReadFromFile_MDF_i4_2d]    Aborting.")');call goErr
          status=1;return
       else 
          write (gol,'("[ReadFromFile_MDF_i4_2d]    Using default type promotion.")'); call goPr
       endif
    endif

    call MDF_Get_Var(hid,vid,buffer,status)
    IF_NOTOK_RETURN(status=1)

    call MDF_Close(hid,status)
    IF_NOTOK_RETURN(status=1)

    status = 0

  end subroutine ReadFromFile_MDF_i4_2d

  subroutine ReadFromFile_MDF_i4_3d(fname,ftype,vname,buffer,lenient_typing,status)

    character(len=*), intent(in)	:: fname  ! file name
    integer, intent(in)			:: ftype  ! MDF_HDF4, MDF_NETCDF, MDF_NETCDF4, MDF_HDF5
    character(len=*), intent(in)	:: vname  ! variable name
    integer(4), intent(out)		:: buffer(:,:,:)
    logical, optional, intent(in)	:: lenient_typing
    integer, intent(out)		:: status

    integer				:: hid
    integer				:: vid
    integer				:: xtype
    integer				:: ndims

    character(len=*), parameter		:: rname = mname//'/ReadFromFile_MDF_i4_3d'

    call MDF_open(fname,ftype,mode=MDF_READ,hid=hid,status=status)
    IF_NOTOK_RETURN(status=1)

    call MDF_Inq_VarID(hid,vname,vid,status)
    IF_NOTOK_RETURN(status=1)

    call MDF_Inquire_Variable(hid,vid,status,xtype=xtype,ndims=ndims)
    IF_NOTOK_RETURN(status=1)
    
    if(ndims /= 3) then
       write (gol,'("[ReadFromFile_MDF_i4_3d]  Variable """,a,""" in file """,a,""":")') trim(vname),trim(fname);call goPr
       write (gol,'("[ReadFromFile_MDF_i4_3d]    Was expecting 3 dimensions but found ",i,".")') ndims;call goPr
       write (gol,'("[ReadFromFile_MDF_i4_3d]    Aborting.")');call goErr
       status=1;return
    endif

    if(xtype /= MDF_INT) then
       write (gol,'("[ReadFromFile_MDF_i4_3d]  Variable """,a,""" in file """,a,""":")') trim(vname),trim(fname);call goPr
       write (gol,'("[ReadFromFile_MDF_i4_3d]    Was expecting """,a,""" type, but found """,a,""" type.")') trim(MDF_DATATYPE_NAME(MDF_INT)),trim(MDF_DATATYPE_NAME(xtype));call goPr
       if(.not. present(lenient_typing)) then
          write (gol,'("[ReadFromFile_MDF_i4_3d]    Aborting.")');call goErr
          status=1;return
       else 
          write (gol,'("[ReadFromFile_MDF_i4_3d]    Using default type promotion.")'); call goPr
       endif
    endif

    call MDF_Get_Var(hid,vid,buffer,status)
    IF_NOTOK_RETURN(status=1)

    call MDF_Close(hid,status)
    IF_NOTOK_RETURN(status=1)

    status = 0

  end subroutine ReadFromFile_MDF_i4_3d

  subroutine ReadFromFile_MDF_i4_4d(fname,ftype,vname,buffer,lenient_typing,status)

    character(len=*), intent(in)	:: fname  ! file name
    integer, intent(in)			:: ftype  ! MDF_HDF4, MDF_NETCDF, MDF_NETCDF4, MDF_HDF5
    character(len=*), intent(in)	:: vname  ! variable name
    integer(4), intent(out)		:: buffer(:,:,:,:)
    logical, optional, intent(in)	:: lenient_typing
    integer, intent(out)		:: status

    integer				:: hid
    integer				:: vid
    integer				:: xtype
    integer				:: ndims

    character(len=*), parameter		:: rname = mname//'/ReadFromFile_MDF_i4_4d'

    call MDF_open(fname,ftype,mode=MDF_READ,hid=hid,status=status)
    IF_NOTOK_RETURN(status=1)

    call MDF_Inq_VarID(hid,vname,vid,status)
    IF_NOTOK_RETURN(status=1)

    call MDF_Inquire_Variable(hid,vid,status,xtype=xtype,ndims=ndims)
    IF_NOTOK_RETURN(status=1)
    
    if(ndims /= 4) then
       write (gol,'("[ReadFromFile_MDF_i4_4d]  Variable """,a,""" in file """,a,""":")') trim(vname),trim(fname);call goPr
       write (gol,'("[ReadFromFile_MDF_i4_4d]    Was expecting 4 dimensions but found ",i,".")') ndims;call goPr
       write (gol,'("[ReadFromFile_MDF_i4_4d]    Aborting.")');call goErr
       status=1;return
    endif

    if(xtype /= MDF_INT) then
       write (gol,'("[ReadFromFile_MDF_i4_4d]  Variable """,a,""" in file """,a,""":")') trim(vname),trim(fname);call goPr
       write (gol,'("[ReadFromFile_MDF_i4_4d]    Was expecting """,a,""" type, but found """,a,""" type.")') trim(MDF_DATATYPE_NAME(MDF_INT)),trim(MDF_DATATYPE_NAME(xtype));call goPr
       if(.not. present(lenient_typing)) then
          write (gol,'("[ReadFromFile_MDF_i4_4d]    Aborting.")');call goErr
          status=1;return
       else 
          write (gol,'("[ReadFromFile_MDF_i4_4d]    Using default type promotion.")'); call goPr
       endif
    endif

    call MDF_Get_Var(hid,vid,buffer,status)
    IF_NOTOK_RETURN(status=1)

    call MDF_Close(hid,status)
    IF_NOTOK_RETURN(status=1)

    status = 0

  end subroutine ReadFromFile_MDF_i4_4d

  subroutine ReadFromFile_MDF_i4_5d(fname,ftype,vname,buffer,lenient_typing,status)

    character(len=*), intent(in)	:: fname  ! file name
    integer, intent(in)			:: ftype  ! MDF_HDF4, MDF_NETCDF, MDF_NETCDF4, MDF_HDF5
    character(len=*), intent(in)	:: vname  ! variable name
    integer(4), intent(out)		:: buffer(:,:,:,:,:)
    logical, optional, intent(in)	:: lenient_typing
    integer, intent(out)		:: status

    integer				:: hid
    integer				:: vid
    integer				:: xtype
    integer				:: ndims

    character(len=*), parameter		:: rname = mname//'/ReadFromFile_MDF_i4_5d'

    call MDF_open(fname,ftype,mode=MDF_READ,hid=hid,status=status)
    IF_NOTOK_RETURN(status=1)

    call MDF_Inq_VarID(hid,vname,vid,status)
    IF_NOTOK_RETURN(status=1)

    call MDF_Inquire_Variable(hid,vid,status,xtype=xtype,ndims=ndims)
    IF_NOTOK_RETURN(status=1)
    
    if(ndims /= 5) then
       write (gol,'("[ReadFromFile_MDF_i4_5d]  Variable """,a,""" in file """,a,""":")') trim(vname),trim(fname);call goPr
       write (gol,'("[ReadFromFile_MDF_i4_5d]    Was expecting 5 dimensions but found ",i,".")') ndims;call goPr
       write (gol,'("[ReadFromFile_MDF_i4_5d]    Aborting.")');call goErr
       status=1;return
    endif

    if(xtype /= MDF_INT) then
       write (gol,'("[ReadFromFile_MDF_i4_5d]  Variable """,a,""" in file """,a,""":")') trim(vname),trim(fname);call goPr
       write (gol,'("[ReadFromFile_MDF_i4_5d]    Was expecting """,a,""" type, but found """,a,""" type.")') trim(MDF_DATATYPE_NAME(MDF_INT)),trim(MDF_DATATYPE_NAME(xtype));call goPr
       if(.not. present(lenient_typing)) then
          write (gol,'("[ReadFromFile_MDF_i4_5d]    Aborting.")');call goErr
          status=1;return
       else 
          write (gol,'("[ReadFromFile_MDF_i4_5d]    Using default type promotion.")'); call goPr
       endif
    endif

    call MDF_Get_Var(hid,vid,buffer,status)
    IF_NOTOK_RETURN(status=1)

    call MDF_Close(hid,status)
    IF_NOTOK_RETURN(status=1)

    status = 0

  end subroutine ReadFromFile_MDF_i4_5d

  subroutine ReadFromFile_MDF_i2_1d(fname,ftype,vname,buffer,lenient_typing,status)

    character(len=*), intent(in)	:: fname  ! file name
    integer, intent(in)			:: ftype  ! MDF_HDF4, MDF_NETCDF, MDF_NETCDF4, MDF_HDF5
    character(len=*), intent(in)	:: vname  ! variable name
    integer(2), intent(out)		:: buffer(:)
    logical, optional, intent(in)	:: lenient_typing
    integer, intent(out)		:: status

    integer				:: hid
    integer				:: vid
    integer				:: xtype
    integer				:: ndims

    character(len=*), parameter		:: rname = mname//'/ReadFromFile_MDF_i2_1d'

    call MDF_open(fname,ftype,mode=MDF_READ,hid=hid,status=status)
    IF_NOTOK_RETURN(status=1)

    call MDF_Inq_VarID(hid,vname,vid,status)
    IF_NOTOK_RETURN(status=1)

    call MDF_Inquire_Variable(hid,vid,status,xtype=xtype,ndims=ndims)
    IF_NOTOK_RETURN(status=1)
    
    if(ndims /= 1) then
       write (gol,'("[ReadFromFile_MDF_i2_1d]  Variable """,a,""" in file """,a,""":")') trim(vname),trim(fname);call goPr
       write (gol,'("[ReadFromFile_MDF_i2_1d]    Was expecting 1 dimensions but found ",i,".")') ndims;call goPr
       write (gol,'("[ReadFromFile_MDF_i2_1d]    Aborting.")');call goErr
       status=1;return
    endif

    if(xtype /= MDF_SHORT) then
       write (gol,'("[ReadFromFile_MDF_i2_1d]  Variable """,a,""" in file """,a,""":")') trim(vname),trim(fname);call goPr
       write (gol,'("[ReadFromFile_MDF_i2_1d]    Was expecting """,a,""" type, but found """,a,""" type.")') trim(MDF_DATATYPE_NAME(MDF_SHORT)),trim(MDF_DATATYPE_NAME(xtype));call goPr
       if(.not. present(lenient_typing)) then
          write (gol,'("[ReadFromFile_MDF_i2_1d]    Aborting.")');call goErr
          status=1;return
       else 
          write (gol,'("[ReadFromFile_MDF_i2_1d]    Using default type promotion.")'); call goPr
       endif
    endif

    call MDF_Get_Var(hid,vid,buffer,status)
    IF_NOTOK_RETURN(status=1)

    call MDF_Close(hid,status)
    IF_NOTOK_RETURN(status=1)

    status = 0

  end subroutine ReadFromFile_MDF_i2_1d

  subroutine ReadFromFile_MDF_i2_2d(fname,ftype,vname,buffer,lenient_typing,status)

    character(len=*), intent(in)	:: fname  ! file name
    integer, intent(in)			:: ftype  ! MDF_HDF4, MDF_NETCDF, MDF_NETCDF4, MDF_HDF5
    character(len=*), intent(in)	:: vname  ! variable name
    integer(2), intent(out)		:: buffer(:,:)
    logical, optional, intent(in)	:: lenient_typing
    integer, intent(out)		:: status

    integer				:: hid
    integer				:: vid
    integer				:: xtype
    integer				:: ndims

    character(len=*), parameter		:: rname = mname//'/ReadFromFile_MDF_i2_2d'

    call MDF_open(fname,ftype,mode=MDF_READ,hid=hid,status=status)
    IF_NOTOK_RETURN(status=1)

    call MDF_Inq_VarID(hid,vname,vid,status)
    IF_NOTOK_RETURN(status=1)

    call MDF_Inquire_Variable(hid,vid,status,xtype=xtype,ndims=ndims)
    IF_NOTOK_RETURN(status=1)
    
    if(ndims /= 2) then
       write (gol,'("[ReadFromFile_MDF_i2_2d]  Variable """,a,""" in file """,a,""":")') trim(vname),trim(fname);call goPr
       write (gol,'("[ReadFromFile_MDF_i2_2d]    Was expecting 2 dimensions but found ",i,".")') ndims;call goPr
       write (gol,'("[ReadFromFile_MDF_i2_2d]    Aborting.")');call goErr
       status=1;return
    endif

    if(xtype /= MDF_SHORT) then
       write (gol,'("[ReadFromFile_MDF_i2_2d]  Variable """,a,""" in file """,a,""":")') trim(vname),trim(fname);call goPr
       write (gol,'("[ReadFromFile_MDF_i2_2d]    Was expecting """,a,""" type, but found """,a,""" type.")') trim(MDF_DATATYPE_NAME(MDF_SHORT)),trim(MDF_DATATYPE_NAME(xtype));call goPr
       if(.not. present(lenient_typing)) then
          write (gol,'("[ReadFromFile_MDF_i2_2d]    Aborting.")');call goErr
          status=1;return
       else 
          write (gol,'("[ReadFromFile_MDF_i2_2d]    Using default type promotion.")'); call goPr
       endif
    endif

    call MDF_Get_Var(hid,vid,buffer,status)
    IF_NOTOK_RETURN(status=1)

    call MDF_Close(hid,status)
    IF_NOTOK_RETURN(status=1)

    status = 0

  end subroutine ReadFromFile_MDF_i2_2d

  subroutine ReadFromFile_MDF_i2_3d(fname,ftype,vname,buffer,lenient_typing,status)

    character(len=*), intent(in)	:: fname  ! file name
    integer, intent(in)			:: ftype  ! MDF_HDF4, MDF_NETCDF, MDF_NETCDF4, MDF_HDF5
    character(len=*), intent(in)	:: vname  ! variable name
    integer(2), intent(out)		:: buffer(:,:,:)
    logical, optional, intent(in)	:: lenient_typing
    integer, intent(out)		:: status

    integer				:: hid
    integer				:: vid
    integer				:: xtype
    integer				:: ndims

    character(len=*), parameter		:: rname = mname//'/ReadFromFile_MDF_i2_3d'

    call MDF_open(fname,ftype,mode=MDF_READ,hid=hid,status=status)
    IF_NOTOK_RETURN(status=1)

    call MDF_Inq_VarID(hid,vname,vid,status)
    IF_NOTOK_RETURN(status=1)

    call MDF_Inquire_Variable(hid,vid,status,xtype=xtype,ndims=ndims)
    IF_NOTOK_RETURN(status=1)
    
    if(ndims /= 3) then
       write (gol,'("[ReadFromFile_MDF_i2_3d]  Variable """,a,""" in file """,a,""":")') trim(vname),trim(fname);call goPr
       write (gol,'("[ReadFromFile_MDF_i2_3d]    Was expecting 3 dimensions but found ",i,".")') ndims;call goPr
       write (gol,'("[ReadFromFile_MDF_i2_3d]    Aborting.")');call goErr
       status=1;return
    endif

    if(xtype /= MDF_SHORT) then
       write (gol,'("[ReadFromFile_MDF_i2_3d]  Variable """,a,""" in file """,a,""":")') trim(vname),trim(fname);call goPr
       write (gol,'("[ReadFromFile_MDF_i2_3d]    Was expecting """,a,""" type, but found """,a,""" type.")') trim(MDF_DATATYPE_NAME(MDF_SHORT)),trim(MDF_DATATYPE_NAME(xtype));call goPr
       if(.not. present(lenient_typing)) then
          write (gol,'("[ReadFromFile_MDF_i2_3d]    Aborting.")');call goErr
          status=1;return
       else 
          write (gol,'("[ReadFromFile_MDF_i2_3d]    Using default type promotion.")'); call goPr
       endif
    endif

    call MDF_Get_Var(hid,vid,buffer,status)
    IF_NOTOK_RETURN(status=1)

    call MDF_Close(hid,status)
    IF_NOTOK_RETURN(status=1)

    status = 0

  end subroutine ReadFromFile_MDF_i2_3d

  subroutine ReadFromFile_MDF_i2_4d(fname,ftype,vname,buffer,lenient_typing,status)

    character(len=*), intent(in)	:: fname  ! file name
    integer, intent(in)			:: ftype  ! MDF_HDF4, MDF_NETCDF, MDF_NETCDF4, MDF_HDF5
    character(len=*), intent(in)	:: vname  ! variable name
    integer(2), intent(out)		:: buffer(:,:,:,:)
    logical, optional, intent(in)	:: lenient_typing
    integer, intent(out)		:: status

    integer				:: hid
    integer				:: vid
    integer				:: xtype
    integer				:: ndims

    character(len=*), parameter		:: rname = mname//'/ReadFromFile_MDF_i2_4d'

    call MDF_open(fname,ftype,mode=MDF_READ,hid=hid,status=status)
    IF_NOTOK_RETURN(status=1)

    call MDF_Inq_VarID(hid,vname,vid,status)
    IF_NOTOK_RETURN(status=1)

    call MDF_Inquire_Variable(hid,vid,status,xtype=xtype,ndims=ndims)
    IF_NOTOK_RETURN(status=1)
    
    if(ndims /= 4) then
       write (gol,'("[ReadFromFile_MDF_i2_4d]  Variable """,a,""" in file """,a,""":")') trim(vname),trim(fname);call goPr
       write (gol,'("[ReadFromFile_MDF_i2_4d]    Was expecting 4 dimensions but found ",i,".")') ndims;call goPr
       write (gol,'("[ReadFromFile_MDF_i2_4d]    Aborting.")');call goErr
       status=1;return
    endif

    if(xtype /= MDF_SHORT) then
       write (gol,'("[ReadFromFile_MDF_i2_4d]  Variable """,a,""" in file """,a,""":")') trim(vname),trim(fname);call goPr
       write (gol,'("[ReadFromFile_MDF_i2_4d]    Was expecting """,a,""" type, but found """,a,""" type.")') trim(MDF_DATATYPE_NAME(MDF_SHORT)),trim(MDF_DATATYPE_NAME(xtype));call goPr
       if(.not. present(lenient_typing)) then
          write (gol,'("[ReadFromFile_MDF_i2_4d]    Aborting.")');call goErr
          status=1;return
       else 
          write (gol,'("[ReadFromFile_MDF_i2_4d]    Using default type promotion.")'); call goPr
       endif
    endif

    call MDF_Get_Var(hid,vid,buffer,status)
    IF_NOTOK_RETURN(status=1)

    call MDF_Close(hid,status)
    IF_NOTOK_RETURN(status=1)

    status = 0

  end subroutine ReadFromFile_MDF_i2_4d

  subroutine ReadFromFile_MDF_i2_5d(fname,ftype,vname,buffer,lenient_typing,status)

    character(len=*), intent(in)	:: fname  ! file name
    integer, intent(in)			:: ftype  ! MDF_HDF4, MDF_NETCDF, MDF_NETCDF4, MDF_HDF5
    character(len=*), intent(in)	:: vname  ! variable name
    integer(2), intent(out)		:: buffer(:,:,:,:,:)
    logical, optional, intent(in)	:: lenient_typing
    integer, intent(out)		:: status

    integer				:: hid
    integer				:: vid
    integer				:: xtype
    integer				:: ndims

    character(len=*), parameter		:: rname = mname//'/ReadFromFile_MDF_i2_5d'

    call MDF_open(fname,ftype,mode=MDF_READ,hid=hid,status=status)
    IF_NOTOK_RETURN(status=1)

    call MDF_Inq_VarID(hid,vname,vid,status)
    IF_NOTOK_RETURN(status=1)

    call MDF_Inquire_Variable(hid,vid,status,xtype=xtype,ndims=ndims)
    IF_NOTOK_RETURN(status=1)
    
    if(ndims /= 5) then
       write (gol,'("[ReadFromFile_MDF_i2_5d]  Variable """,a,""" in file """,a,""":")') trim(vname),trim(fname);call goPr
       write (gol,'("[ReadFromFile_MDF_i2_5d]    Was expecting 5 dimensions but found ",i,".")') ndims;call goPr
       write (gol,'("[ReadFromFile_MDF_i2_5d]    Aborting.")');call goErr
       status=1;return
    endif

    if(xtype /= MDF_SHORT) then
       write (gol,'("[ReadFromFile_MDF_i2_5d]  Variable """,a,""" in file """,a,""":")') trim(vname),trim(fname);call goPr
       write (gol,'("[ReadFromFile_MDF_i2_5d]    Was expecting """,a,""" type, but found """,a,""" type.")') trim(MDF_DATATYPE_NAME(MDF_SHORT)),trim(MDF_DATATYPE_NAME(xtype));call goPr
       if(.not. present(lenient_typing)) then
          write (gol,'("[ReadFromFile_MDF_i2_5d]    Aborting.")');call goErr
          status=1;return
       else 
          write (gol,'("[ReadFromFile_MDF_i2_5d]    Using default type promotion.")'); call goPr
       endif
    endif

    call MDF_Get_Var(hid,vid,buffer,status)
    IF_NOTOK_RETURN(status=1)

    call MDF_Close(hid,status)
    IF_NOTOK_RETURN(status=1)

    status = 0

  end subroutine ReadFromFile_MDF_i2_5d

  subroutine ReadFromFile_MDF_i8_1d(fname,ftype,vname,buffer,lenient_typing,status)

    character(len=*), intent(in)	:: fname  ! file name
    integer, intent(in)			:: ftype  ! MDF_HDF4, MDF_NETCDF, MDF_NETCDF4, MDF_HDF5
    character(len=*), intent(in)	:: vname  ! variable name
    integer(8), intent(out)		:: buffer(:)
    logical, optional, intent(in)	:: lenient_typing
    integer, intent(out)		:: status

    integer				:: hid
    integer				:: vid
    integer				:: xtype
    integer				:: ndims

    character(len=*), parameter		:: rname = mname//'/ReadFromFile_MDF_i8_1d'

    call MDF_open(fname,ftype,mode=MDF_READ,hid=hid,status=status)
    IF_NOTOK_RETURN(status=1)

    call MDF_Inq_VarID(hid,vname,vid,status)
    IF_NOTOK_RETURN(status=1)

    call MDF_Inquire_Variable(hid,vid,status,xtype=xtype,ndims=ndims)
    IF_NOTOK_RETURN(status=1)
    
    if(ndims /= 1) then
       write (gol,'("[ReadFromFile_MDF_i8_1d]  Variable """,a,""" in file """,a,""":")') trim(vname),trim(fname);call goPr
       write (gol,'("[ReadFromFile_MDF_i8_1d]    Was expecting 1 dimensions but found ",i,".")') ndims;call goPr
       write (gol,'("[ReadFromFile_MDF_i8_1d]    Aborting.")');call goErr
       status=1;return
    endif

    if(xtype /= MDF_INT64) then
       write (gol,'("[ReadFromFile_MDF_i8_1d]  Variable """,a,""" in file """,a,""":")') trim(vname),trim(fname);call goPr
       write (gol,'("[ReadFromFile_MDF_i8_1d]    Was expecting """,a,""" type, but found """,a,""" type.")') trim(MDF_DATATYPE_NAME(MDF_INT64)),trim(MDF_DATATYPE_NAME(xtype));call goPr
       if(.not. present(lenient_typing)) then
          write (gol,'("[ReadFromFile_MDF_i8_1d]    Aborting.")');call goErr
          status=1;return
       else 
          write (gol,'("[ReadFromFile_MDF_i8_1d]    Using default type promotion.")'); call goPr
       endif
    endif

    call MDF_Get_Var(hid,vid,buffer,status)
    IF_NOTOK_RETURN(status=1)

    call MDF_Close(hid,status)
    IF_NOTOK_RETURN(status=1)

    status = 0

  end subroutine ReadFromFile_MDF_i8_1d

  subroutine ReadFromFile_MDF_i8_2d(fname,ftype,vname,buffer,lenient_typing,status)

    character(len=*), intent(in)	:: fname  ! file name
    integer, intent(in)			:: ftype  ! MDF_HDF4, MDF_NETCDF, MDF_NETCDF4, MDF_HDF5
    character(len=*), intent(in)	:: vname  ! variable name
    integer(8), intent(out)		:: buffer(:,:)
    logical, optional, intent(in)	:: lenient_typing
    integer, intent(out)		:: status

    integer				:: hid
    integer				:: vid
    integer				:: xtype
    integer				:: ndims

    character(len=*), parameter		:: rname = mname//'/ReadFromFile_MDF_i8_2d'

    call MDF_open(fname,ftype,mode=MDF_READ,hid=hid,status=status)
    IF_NOTOK_RETURN(status=1)

    call MDF_Inq_VarID(hid,vname,vid,status)
    IF_NOTOK_RETURN(status=1)

    call MDF_Inquire_Variable(hid,vid,status,xtype=xtype,ndims=ndims)
    IF_NOTOK_RETURN(status=1)
    
    if(ndims /= 2) then
       write (gol,'("[ReadFromFile_MDF_i8_2d]  Variable """,a,""" in file """,a,""":")') trim(vname),trim(fname);call goPr
       write (gol,'("[ReadFromFile_MDF_i8_2d]    Was expecting 2 dimensions but found ",i,".")') ndims;call goPr
       write (gol,'("[ReadFromFile_MDF_i8_2d]    Aborting.")');call goErr
       status=1;return
    endif

    if(xtype /= MDF_INT64) then
       write (gol,'("[ReadFromFile_MDF_i8_2d]  Variable """,a,""" in file """,a,""":")') trim(vname),trim(fname);call goPr
       write (gol,'("[ReadFromFile_MDF_i8_2d]    Was expecting """,a,""" type, but found """,a,""" type.")') trim(MDF_DATATYPE_NAME(MDF_INT64)),trim(MDF_DATATYPE_NAME(xtype));call goPr
       if(.not. present(lenient_typing)) then
          write (gol,'("[ReadFromFile_MDF_i8_2d]    Aborting.")');call goErr
          status=1;return
       else 
          write (gol,'("[ReadFromFile_MDF_i8_2d]    Using default type promotion.")'); call goPr
       endif
    endif

    call MDF_Get_Var(hid,vid,buffer,status)
    IF_NOTOK_RETURN(status=1)

    call MDF_Close(hid,status)
    IF_NOTOK_RETURN(status=1)

    status = 0

  end subroutine ReadFromFile_MDF_i8_2d

  subroutine ReadFromFile_MDF_i8_3d(fname,ftype,vname,buffer,lenient_typing,status)

    character(len=*), intent(in)	:: fname  ! file name
    integer, intent(in)			:: ftype  ! MDF_HDF4, MDF_NETCDF, MDF_NETCDF4, MDF_HDF5
    character(len=*), intent(in)	:: vname  ! variable name
    integer(8), intent(out)		:: buffer(:,:,:)
    logical, optional, intent(in)	:: lenient_typing
    integer, intent(out)		:: status

    integer				:: hid
    integer				:: vid
    integer				:: xtype
    integer				:: ndims

    character(len=*), parameter		:: rname = mname//'/ReadFromFile_MDF_i8_3d'

    call MDF_open(fname,ftype,mode=MDF_READ,hid=hid,status=status)
    IF_NOTOK_RETURN(status=1)

    call MDF_Inq_VarID(hid,vname,vid,status)
    IF_NOTOK_RETURN(status=1)

    call MDF_Inquire_Variable(hid,vid,status,xtype=xtype,ndims=ndims)
    IF_NOTOK_RETURN(status=1)
    
    if(ndims /= 3) then
       write (gol,'("[ReadFromFile_MDF_i8_3d]  Variable """,a,""" in file """,a,""":")') trim(vname),trim(fname);call goPr
       write (gol,'("[ReadFromFile_MDF_i8_3d]    Was expecting 3 dimensions but found ",i,".")') ndims;call goPr
       write (gol,'("[ReadFromFile_MDF_i8_3d]    Aborting.")');call goErr
       status=1;return
    endif

    if(xtype /= MDF_INT64) then
       write (gol,'("[ReadFromFile_MDF_i8_3d]  Variable """,a,""" in file """,a,""":")') trim(vname),trim(fname);call goPr
       write (gol,'("[ReadFromFile_MDF_i8_3d]    Was expecting """,a,""" type, but found """,a,""" type.")') trim(MDF_DATATYPE_NAME(MDF_INT64)),trim(MDF_DATATYPE_NAME(xtype));call goPr
       if(.not. present(lenient_typing)) then
          write (gol,'("[ReadFromFile_MDF_i8_3d]    Aborting.")');call goErr
          status=1;return
       else 
          write (gol,'("[ReadFromFile_MDF_i8_3d]    Using default type promotion.")'); call goPr
       endif
    endif

    call MDF_Get_Var(hid,vid,buffer,status)
    IF_NOTOK_RETURN(status=1)

    call MDF_Close(hid,status)
    IF_NOTOK_RETURN(status=1)

    status = 0

  end subroutine ReadFromFile_MDF_i8_3d

  subroutine ReadFromFile_MDF_i8_4d(fname,ftype,vname,buffer,lenient_typing,status)

    character(len=*), intent(in)	:: fname  ! file name
    integer, intent(in)			:: ftype  ! MDF_HDF4, MDF_NETCDF, MDF_NETCDF4, MDF_HDF5
    character(len=*), intent(in)	:: vname  ! variable name
    integer(8), intent(out)		:: buffer(:,:,:,:)
    logical, optional, intent(in)	:: lenient_typing
    integer, intent(out)		:: status

    integer				:: hid
    integer				:: vid
    integer				:: xtype
    integer				:: ndims

    character(len=*), parameter		:: rname = mname//'/ReadFromFile_MDF_i8_4d'

    call MDF_open(fname,ftype,mode=MDF_READ,hid=hid,status=status)
    IF_NOTOK_RETURN(status=1)

    call MDF_Inq_VarID(hid,vname,vid,status)
    IF_NOTOK_RETURN(status=1)

    call MDF_Inquire_Variable(hid,vid,status,xtype=xtype,ndims=ndims)
    IF_NOTOK_RETURN(status=1)
    
    if(ndims /= 4) then
       write (gol,'("[ReadFromFile_MDF_i8_4d]  Variable """,a,""" in file """,a,""":")') trim(vname),trim(fname);call goPr
       write (gol,'("[ReadFromFile_MDF_i8_4d]    Was expecting 4 dimensions but found ",i,".")') ndims;call goPr
       write (gol,'("[ReadFromFile_MDF_i8_4d]    Aborting.")');call goErr
       status=1;return
    endif

    if(xtype /= MDF_INT64) then
       write (gol,'("[ReadFromFile_MDF_i8_4d]  Variable """,a,""" in file """,a,""":")') trim(vname),trim(fname);call goPr
       write (gol,'("[ReadFromFile_MDF_i8_4d]    Was expecting """,a,""" type, but found """,a,""" type.")') trim(MDF_DATATYPE_NAME(MDF_INT64)),trim(MDF_DATATYPE_NAME(xtype));call goPr
       if(.not. present(lenient_typing)) then
          write (gol,'("[ReadFromFile_MDF_i8_4d]    Aborting.")');call goErr
          status=1;return
       else 
          write (gol,'("[ReadFromFile_MDF_i8_4d]    Using default type promotion.")'); call goPr
       endif
    endif

    call MDF_Get_Var(hid,vid,buffer,status)
    IF_NOTOK_RETURN(status=1)

    call MDF_Close(hid,status)
    IF_NOTOK_RETURN(status=1)

    status = 0

  end subroutine ReadFromFile_MDF_i8_4d

  subroutine ReadFromFile_MDF_i8_5d(fname,ftype,vname,buffer,lenient_typing,status)

    character(len=*), intent(in)	:: fname  ! file name
    integer, intent(in)			:: ftype  ! MDF_HDF4, MDF_NETCDF, MDF_NETCDF4, MDF_HDF5
    character(len=*), intent(in)	:: vname  ! variable name
    integer(8), intent(out)		:: buffer(:,:,:,:,:)
    logical, optional, intent(in)	:: lenient_typing
    integer, intent(out)		:: status

    integer				:: hid
    integer				:: vid
    integer				:: xtype
    integer				:: ndims

    character(len=*), parameter		:: rname = mname//'/ReadFromFile_MDF_i8_5d'

    call MDF_open(fname,ftype,mode=MDF_READ,hid=hid,status=status)
    IF_NOTOK_RETURN(status=1)

    call MDF_Inq_VarID(hid,vname,vid,status)
    IF_NOTOK_RETURN(status=1)

    call MDF_Inquire_Variable(hid,vid,status,xtype=xtype,ndims=ndims)
    IF_NOTOK_RETURN(status=1)
    
    if(ndims /= 5) then
       write (gol,'("[ReadFromFile_MDF_i8_5d]  Variable """,a,""" in file """,a,""":")') trim(vname),trim(fname);call goPr
       write (gol,'("[ReadFromFile_MDF_i8_5d]    Was expecting 5 dimensions but found ",i,".")') ndims;call goPr
       write (gol,'("[ReadFromFile_MDF_i8_5d]    Aborting.")');call goErr
       status=1;return
    endif

    if(xtype /= MDF_INT64) then
       write (gol,'("[ReadFromFile_MDF_i8_5d]  Variable """,a,""" in file """,a,""":")') trim(vname),trim(fname);call goPr
       write (gol,'("[ReadFromFile_MDF_i8_5d]    Was expecting """,a,""" type, but found """,a,""" type.")') trim(MDF_DATATYPE_NAME(MDF_INT64)),trim(MDF_DATATYPE_NAME(xtype));call goPr
       if(.not. present(lenient_typing)) then
          write (gol,'("[ReadFromFile_MDF_i8_5d]    Aborting.")');call goErr
          status=1;return
       else 
          write (gol,'("[ReadFromFile_MDF_i8_5d]    Using default type promotion.")'); call goPr
       endif
    endif

    call MDF_Get_Var(hid,vid,buffer,status)
    IF_NOTOK_RETURN(status=1)

    call MDF_Close(hid,status)
    IF_NOTOK_RETURN(status=1)

    status = 0

  end subroutine ReadFromFile_MDF_i8_5d

  subroutine ReadFromFile_MDF_r8_1d(fname,ftype,vname,buffer,lenient_typing,status)

    character(len=*), intent(in)	:: fname  ! file name
    integer, intent(in)			:: ftype  ! MDF_HDF4, MDF_NETCDF, MDF_NETCDF4, MDF_HDF5
    character(len=*), intent(in)	:: vname  ! variable name
    real(8), intent(out)		:: buffer(:)
    logical, optional, intent(in)	:: lenient_typing
    integer, intent(out)		:: status

    integer				:: hid
    integer				:: vid
    integer				:: xtype
    integer				:: ndims

    character(len=*), parameter		:: rname = mname//'/ReadFromFile_MDF_r8_1d'

    call MDF_open(fname,ftype,mode=MDF_READ,hid=hid,status=status)
    IF_NOTOK_RETURN(status=1)

    call MDF_Inq_VarID(hid,vname,vid,status)
    IF_NOTOK_RETURN(status=1)

    call MDF_Inquire_Variable(hid,vid,status,xtype=xtype,ndims=ndims)
    IF_NOTOK_RETURN(status=1)
    
    if(ndims /= 1) then
       write (gol,'("[ReadFromFile_MDF_r8_1d]  Variable """,a,""" in file """,a,""":")') trim(vname),trim(fname);call goPr
       write (gol,'("[ReadFromFile_MDF_r8_1d]    Was expecting 1 dimensions but found ",i,".")') ndims;call goPr
       write (gol,'("[ReadFromFile_MDF_r8_1d]    Aborting.")');call goErr
       status=1;return
    endif

    if(xtype /= MDF_DOUBLE) then
       write (gol,'("[ReadFromFile_MDF_r8_1d]  Variable """,a,""" in file """,a,""":")') trim(vname),trim(fname);call goPr
       write (gol,'("[ReadFromFile_MDF_r8_1d]    Was expecting """,a,""" type, but found """,a,""" type.")') trim(MDF_DATATYPE_NAME(MDF_DOUBLE)),trim(MDF_DATATYPE_NAME(xtype));call goPr
       if(.not. present(lenient_typing)) then
          write (gol,'("[ReadFromFile_MDF_r8_1d]    Aborting.")');call goErr
          status=1;return
       else 
          write (gol,'("[ReadFromFile_MDF_r8_1d]    Using default type promotion.")'); call goPr
       endif
    endif

    call MDF_Get_Var(hid,vid,buffer,status)
    IF_NOTOK_RETURN(status=1)

    call MDF_Close(hid,status)
    IF_NOTOK_RETURN(status=1)

    status = 0

  end subroutine ReadFromFile_MDF_r8_1d

  subroutine ReadFromFile_MDF_r8_2d(fname,ftype,vname,buffer,lenient_typing,status)

    character(len=*), intent(in)	:: fname  ! file name
    integer, intent(in)			:: ftype  ! MDF_HDF4, MDF_NETCDF, MDF_NETCDF4, MDF_HDF5
    character(len=*), intent(in)	:: vname  ! variable name
    real(8), intent(out)		:: buffer(:,:)
    logical, optional, intent(in)	:: lenient_typing
    integer, intent(out)		:: status

    integer				:: hid
    integer				:: vid
    integer				:: xtype
    integer				:: ndims

    character(len=*), parameter		:: rname = mname//'/ReadFromFile_MDF_r8_2d'

    call MDF_open(fname,ftype,mode=MDF_READ,hid=hid,status=status)
    IF_NOTOK_RETURN(status=1)

    call MDF_Inq_VarID(hid,vname,vid,status)
    IF_NOTOK_RETURN(status=1)

    call MDF_Inquire_Variable(hid,vid,status,xtype=xtype,ndims=ndims)
    IF_NOTOK_RETURN(status=1)
    
    if(ndims /= 2) then
       write (gol,'("[ReadFromFile_MDF_r8_2d]  Variable """,a,""" in file """,a,""":")') trim(vname),trim(fname);call goPr
       write (gol,'("[ReadFromFile_MDF_r8_2d]    Was expecting 2 dimensions but found ",i,".")') ndims;call goPr
       write (gol,'("[ReadFromFile_MDF_r8_2d]    Aborting.")');call goErr
       status=1;return
    endif

    if(xtype /= MDF_DOUBLE) then
       write (gol,'("[ReadFromFile_MDF_r8_2d]  Variable """,a,""" in file """,a,""":")') trim(vname),trim(fname);call goPr
       write (gol,'("[ReadFromFile_MDF_r8_2d]    Was expecting """,a,""" type, but found """,a,""" type.")') trim(MDF_DATATYPE_NAME(MDF_DOUBLE)),trim(MDF_DATATYPE_NAME(xtype));call goPr
       if(.not. present(lenient_typing)) then
          write (gol,'("[ReadFromFile_MDF_r8_2d]    Aborting.")');call goErr
          status=1;return
       else 
          write (gol,'("[ReadFromFile_MDF_r8_2d]    Using default type promotion.")'); call goPr
       endif
    endif

    call MDF_Get_Var(hid,vid,buffer,status)
    IF_NOTOK_RETURN(status=1)

    call MDF_Close(hid,status)
    IF_NOTOK_RETURN(status=1)

    status = 0

  end subroutine ReadFromFile_MDF_r8_2d

  subroutine ReadFromFile_MDF_r8_3d(fname,ftype,vname,buffer,lenient_typing,status)

    character(len=*), intent(in)	:: fname  ! file name
    integer, intent(in)			:: ftype  ! MDF_HDF4, MDF_NETCDF, MDF_NETCDF4, MDF_HDF5
    character(len=*), intent(in)	:: vname  ! variable name
    real(8), intent(out)		:: buffer(:,:,:)
    logical, optional, intent(in)	:: lenient_typing
    integer, intent(out)		:: status

    integer				:: hid
    integer				:: vid
    integer				:: xtype
    integer				:: ndims

    character(len=*), parameter		:: rname = mname//'/ReadFromFile_MDF_r8_3d'

    call MDF_open(fname,ftype,mode=MDF_READ,hid=hid,status=status)
    IF_NOTOK_RETURN(status=1)

    call MDF_Inq_VarID(hid,vname,vid,status)
    IF_NOTOK_RETURN(status=1)

    call MDF_Inquire_Variable(hid,vid,status,xtype=xtype,ndims=ndims)
    IF_NOTOK_RETURN(status=1)
    
    if(ndims /= 3) then
       write (gol,'("[ReadFromFile_MDF_r8_3d]  Variable """,a,""" in file """,a,""":")') trim(vname),trim(fname);call goPr
       write (gol,'("[ReadFromFile_MDF_r8_3d]    Was expecting 3 dimensions but found ",i,".")') ndims;call goPr
       write (gol,'("[ReadFromFile_MDF_r8_3d]    Aborting.")');call goErr
       status=1;return
    endif

    if(xtype /= MDF_DOUBLE) then
       write (gol,'("[ReadFromFile_MDF_r8_3d]  Variable """,a,""" in file """,a,""":")') trim(vname),trim(fname);call goPr
       write (gol,'("[ReadFromFile_MDF_r8_3d]    Was expecting """,a,""" type, but found """,a,""" type.")') trim(MDF_DATATYPE_NAME(MDF_DOUBLE)),trim(MDF_DATATYPE_NAME(xtype));call goPr
       if(.not. present(lenient_typing)) then
          write (gol,'("[ReadFromFile_MDF_r8_3d]    Aborting.")');call goErr
          status=1;return
       else 
          write (gol,'("[ReadFromFile_MDF_r8_3d]    Using default type promotion.")'); call goPr
       endif
    endif

    call MDF_Get_Var(hid,vid,buffer,status)
    IF_NOTOK_RETURN(status=1)

    call MDF_Close(hid,status)
    IF_NOTOK_RETURN(status=1)

    status = 0

  end subroutine ReadFromFile_MDF_r8_3d

  subroutine ReadFromFile_MDF_r8_4d(fname,ftype,vname,buffer,lenient_typing,status)

    character(len=*), intent(in)	:: fname  ! file name
    integer, intent(in)			:: ftype  ! MDF_HDF4, MDF_NETCDF, MDF_NETCDF4, MDF_HDF5
    character(len=*), intent(in)	:: vname  ! variable name
    real(8), intent(out)		:: buffer(:,:,:,:)
    logical, optional, intent(in)	:: lenient_typing
    integer, intent(out)		:: status

    integer				:: hid
    integer				:: vid
    integer				:: xtype
    integer				:: ndims

    character(len=*), parameter		:: rname = mname//'/ReadFromFile_MDF_r8_4d'

    call MDF_open(fname,ftype,mode=MDF_READ,hid=hid,status=status)
    IF_NOTOK_RETURN(status=1)

    call MDF_Inq_VarID(hid,vname,vid,status)
    IF_NOTOK_RETURN(status=1)

    call MDF_Inquire_Variable(hid,vid,status,xtype=xtype,ndims=ndims)
    IF_NOTOK_RETURN(status=1)
    
    if(ndims /= 4) then
       write (gol,'("[ReadFromFile_MDF_r8_4d]  Variable """,a,""" in file """,a,""":")') trim(vname),trim(fname);call goPr
       write (gol,'("[ReadFromFile_MDF_r8_4d]    Was expecting 4 dimensions but found ",i,".")') ndims;call goPr
       write (gol,'("[ReadFromFile_MDF_r8_4d]    Aborting.")');call goErr
       status=1;return
    endif

    if(xtype /= MDF_DOUBLE) then
       write (gol,'("[ReadFromFile_MDF_r8_4d]  Variable """,a,""" in file """,a,""":")') trim(vname),trim(fname);call goPr
       write (gol,'("[ReadFromFile_MDF_r8_4d]    Was expecting """,a,""" type, but found """,a,""" type.")') trim(MDF_DATATYPE_NAME(MDF_DOUBLE)),trim(MDF_DATATYPE_NAME(xtype));call goPr
       if(.not. present(lenient_typing)) then
          write (gol,'("[ReadFromFile_MDF_r8_4d]    Aborting.")');call goErr
          status=1;return
       else 
          write (gol,'("[ReadFromFile_MDF_r8_4d]    Using default type promotion.")'); call goPr
       endif
    endif

    call MDF_Get_Var(hid,vid,buffer,status)
    IF_NOTOK_RETURN(status=1)

    call MDF_Close(hid,status)
    IF_NOTOK_RETURN(status=1)

    status = 0

  end subroutine ReadFromFile_MDF_r8_4d

  subroutine ReadFromFile_MDF_r8_5d(fname,ftype,vname,buffer,lenient_typing,status)

    character(len=*), intent(in)	:: fname  ! file name
    integer, intent(in)			:: ftype  ! MDF_HDF4, MDF_NETCDF, MDF_NETCDF4, MDF_HDF5
    character(len=*), intent(in)	:: vname  ! variable name
    real(8), intent(out)		:: buffer(:,:,:,:,:)
    logical, optional, intent(in)	:: lenient_typing
    integer, intent(out)		:: status

    integer				:: hid
    integer				:: vid
    integer				:: xtype
    integer				:: ndims

    character(len=*), parameter		:: rname = mname//'/ReadFromFile_MDF_r8_5d'

    call MDF_open(fname,ftype,mode=MDF_READ,hid=hid,status=status)
    IF_NOTOK_RETURN(status=1)

    call MDF_Inq_VarID(hid,vname,vid,status)
    IF_NOTOK_RETURN(status=1)

    call MDF_Inquire_Variable(hid,vid,status,xtype=xtype,ndims=ndims)
    IF_NOTOK_RETURN(status=1)
    
    if(ndims /= 5) then
       write (gol,'("[ReadFromFile_MDF_r8_5d]  Variable """,a,""" in file """,a,""":")') trim(vname),trim(fname);call goPr
       write (gol,'("[ReadFromFile_MDF_r8_5d]    Was expecting 5 dimensions but found ",i,".")') ndims;call goPr
       write (gol,'("[ReadFromFile_MDF_r8_5d]    Aborting.")');call goErr
       status=1;return
    endif

    if(xtype /= MDF_DOUBLE) then
       write (gol,'("[ReadFromFile_MDF_r8_5d]  Variable """,a,""" in file """,a,""":")') trim(vname),trim(fname);call goPr
       write (gol,'("[ReadFromFile_MDF_r8_5d]    Was expecting """,a,""" type, but found """,a,""" type.")') trim(MDF_DATATYPE_NAME(MDF_DOUBLE)),trim(MDF_DATATYPE_NAME(xtype));call goPr
       if(.not. present(lenient_typing)) then
          write (gol,'("[ReadFromFile_MDF_r8_5d]    Aborting.")');call goErr
          status=1;return
       else 
          write (gol,'("[ReadFromFile_MDF_r8_5d]    Using default type promotion.")'); call goPr
       endif
    endif

    call MDF_Get_Var(hid,vid,buffer,status)
    IF_NOTOK_RETURN(status=1)

    call MDF_Close(hid,status)
    IF_NOTOK_RETURN(status=1)

    status = 0

  end subroutine ReadFromFile_MDF_r8_5d

  subroutine ReadFromFile_MDF_r4_1d(fname,ftype,vname,buffer,lenient_typing,status)

    character(len=*), intent(in)	:: fname  ! file name
    integer, intent(in)			:: ftype  ! MDF_HDF4, MDF_NETCDF, MDF_NETCDF4, MDF_HDF5
    character(len=*), intent(in)	:: vname  ! variable name
    real(4), intent(out)		:: buffer(:)
    logical, optional, intent(in)	:: lenient_typing
    integer, intent(out)		:: status

    integer				:: hid
    integer				:: vid
    integer				:: xtype
    integer				:: ndims

    character(len=*), parameter		:: rname = mname//'/ReadFromFile_MDF_r4_1d'

    call MDF_open(fname,ftype,mode=MDF_READ,hid=hid,status=status)
    IF_NOTOK_RETURN(status=1)

    call MDF_Inq_VarID(hid,vname,vid,status)
    IF_NOTOK_RETURN(status=1)

    call MDF_Inquire_Variable(hid,vid,status,xtype=xtype,ndims=ndims)
    IF_NOTOK_RETURN(status=1)
    
    if(ndims /= 1) then
       write (gol,'("[ReadFromFile_MDF_r4_1d]  Variable """,a,""" in file """,a,""":")') trim(vname),trim(fname);call goPr
       write (gol,'("[ReadFromFile_MDF_r4_1d]    Was expecting 1 dimensions but found ",i,".")') ndims;call goPr
       write (gol,'("[ReadFromFile_MDF_r4_1d]    Aborting.")');call goErr
       status=1;return
    endif

    if(xtype /= MDF_FLOAT) then
       write (gol,'("[ReadFromFile_MDF_r4_1d]  Variable """,a,""" in file """,a,""":")') trim(vname),trim(fname);call goPr
       write (gol,'("[ReadFromFile_MDF_r4_1d]    Was expecting """,a,""" type, but found """,a,""" type.")') trim(MDF_DATATYPE_NAME(MDF_FLOAT)),trim(MDF_DATATYPE_NAME(xtype));call goPr
       if(.not. present(lenient_typing)) then
          write (gol,'("[ReadFromFile_MDF_r4_1d]    Aborting.")');call goErr
          status=1;return
       else 
          write (gol,'("[ReadFromFile_MDF_r4_1d]    Using default type promotion.")'); call goPr
       endif
    endif

    call MDF_Get_Var(hid,vid,buffer,status)
    IF_NOTOK_RETURN(status=1)

    call MDF_Close(hid,status)
    IF_NOTOK_RETURN(status=1)

    status = 0

  end subroutine ReadFromFile_MDF_r4_1d

  subroutine ReadFromFile_MDF_r4_2d(fname,ftype,vname,buffer,lenient_typing,status)

    character(len=*), intent(in)	:: fname  ! file name
    integer, intent(in)			:: ftype  ! MDF_HDF4, MDF_NETCDF, MDF_NETCDF4, MDF_HDF5
    character(len=*), intent(in)	:: vname  ! variable name
    real(4), intent(out)		:: buffer(:,:)
    logical, optional, intent(in)	:: lenient_typing
    integer, intent(out)		:: status

    integer				:: hid
    integer				:: vid
    integer				:: xtype
    integer				:: ndims

    character(len=*), parameter		:: rname = mname//'/ReadFromFile_MDF_r4_2d'

    call MDF_open(fname,ftype,mode=MDF_READ,hid=hid,status=status)
    IF_NOTOK_RETURN(status=1)

    call MDF_Inq_VarID(hid,vname,vid,status)
    IF_NOTOK_RETURN(status=1)

    call MDF_Inquire_Variable(hid,vid,status,xtype=xtype,ndims=ndims)
    IF_NOTOK_RETURN(status=1)
    
    if(ndims /= 2) then
       write (gol,'("[ReadFromFile_MDF_r4_2d]  Variable """,a,""" in file """,a,""":")') trim(vname),trim(fname);call goPr
       write (gol,'("[ReadFromFile_MDF_r4_2d]    Was expecting 2 dimensions but found ",i,".")') ndims;call goPr
       write (gol,'("[ReadFromFile_MDF_r4_2d]    Aborting.")');call goErr
       status=1;return
    endif

    if(xtype /= MDF_FLOAT) then
       write (gol,'("[ReadFromFile_MDF_r4_2d]  Variable """,a,""" in file """,a,""":")') trim(vname),trim(fname);call goPr
       write (gol,'("[ReadFromFile_MDF_r4_2d]    Was expecting """,a,""" type, but found """,a,""" type.")') trim(MDF_DATATYPE_NAME(MDF_FLOAT)),trim(MDF_DATATYPE_NAME(xtype));call goPr
       if(.not. present(lenient_typing)) then
          write (gol,'("[ReadFromFile_MDF_r4_2d]    Aborting.")');call goErr
          status=1;return
       else 
          write (gol,'("[ReadFromFile_MDF_r4_2d]    Using default type promotion.")'); call goPr
       endif
    endif

    call MDF_Get_Var(hid,vid,buffer,status)
    IF_NOTOK_RETURN(status=1)

    call MDF_Close(hid,status)
    IF_NOTOK_RETURN(status=1)

    status = 0

  end subroutine ReadFromFile_MDF_r4_2d

  subroutine ReadFromFile_MDF_r4_3d(fname,ftype,vname,buffer,lenient_typing,status)

    character(len=*), intent(in)	:: fname  ! file name
    integer, intent(in)			:: ftype  ! MDF_HDF4, MDF_NETCDF, MDF_NETCDF4, MDF_HDF5
    character(len=*), intent(in)	:: vname  ! variable name
    real(4), intent(out)		:: buffer(:,:,:)
    logical, optional, intent(in)	:: lenient_typing
    integer, intent(out)		:: status

    integer				:: hid
    integer				:: vid
    integer				:: xtype
    integer				:: ndims

    character(len=*), parameter		:: rname = mname//'/ReadFromFile_MDF_r4_3d'

    call MDF_open(fname,ftype,mode=MDF_READ,hid=hid,status=status)
    IF_NOTOK_RETURN(status=1)

    call MDF_Inq_VarID(hid,vname,vid,status)
    IF_NOTOK_RETURN(status=1)

    call MDF_Inquire_Variable(hid,vid,status,xtype=xtype,ndims=ndims)
    IF_NOTOK_RETURN(status=1)
    
    if(ndims /= 3) then
       write (gol,'("[ReadFromFile_MDF_r4_3d]  Variable """,a,""" in file """,a,""":")') trim(vname),trim(fname);call goPr
       write (gol,'("[ReadFromFile_MDF_r4_3d]    Was expecting 3 dimensions but found ",i,".")') ndims;call goPr
       write (gol,'("[ReadFromFile_MDF_r4_3d]    Aborting.")');call goErr
       status=1;return
    endif

    if(xtype /= MDF_FLOAT) then
       write (gol,'("[ReadFromFile_MDF_r4_3d]  Variable """,a,""" in file """,a,""":")') trim(vname),trim(fname);call goPr
       write (gol,'("[ReadFromFile_MDF_r4_3d]    Was expecting """,a,""" type, but found """,a,""" type.")') trim(MDF_DATATYPE_NAME(MDF_FLOAT)),trim(MDF_DATATYPE_NAME(xtype));call goPr
       if(.not. present(lenient_typing)) then
          write (gol,'("[ReadFromFile_MDF_r4_3d]    Aborting.")');call goErr
          status=1;return
       else 
          write (gol,'("[ReadFromFile_MDF_r4_3d]    Using default type promotion.")'); call goPr
       endif
    endif

    call MDF_Get_Var(hid,vid,buffer,status)
    IF_NOTOK_RETURN(status=1)

    call MDF_Close(hid,status)
    IF_NOTOK_RETURN(status=1)

    status = 0

  end subroutine ReadFromFile_MDF_r4_3d

  subroutine ReadFromFile_MDF_r4_4d(fname,ftype,vname,buffer,lenient_typing,status)

    character(len=*), intent(in)	:: fname  ! file name
    integer, intent(in)			:: ftype  ! MDF_HDF4, MDF_NETCDF, MDF_NETCDF4, MDF_HDF5
    character(len=*), intent(in)	:: vname  ! variable name
    real(4), intent(out)		:: buffer(:,:,:,:)
    logical, optional, intent(in)	:: lenient_typing
    integer, intent(out)		:: status

    integer				:: hid
    integer				:: vid
    integer				:: xtype
    integer				:: ndims

    character(len=*), parameter		:: rname = mname//'/ReadFromFile_MDF_r4_4d'

    call MDF_open(fname,ftype,mode=MDF_READ,hid=hid,status=status)
    IF_NOTOK_RETURN(status=1)

    call MDF_Inq_VarID(hid,vname,vid,status)
    IF_NOTOK_RETURN(status=1)

    call MDF_Inquire_Variable(hid,vid,status,xtype=xtype,ndims=ndims)
    IF_NOTOK_RETURN(status=1)
    
    if(ndims /= 4) then
       write (gol,'("[ReadFromFile_MDF_r4_4d]  Variable """,a,""" in file """,a,""":")') trim(vname),trim(fname);call goPr
       write (gol,'("[ReadFromFile_MDF_r4_4d]    Was expecting 4 dimensions but found ",i,".")') ndims;call goPr
       write (gol,'("[ReadFromFile_MDF_r4_4d]    Aborting.")');call goErr
       status=1;return
    endif

    if(xtype /= MDF_FLOAT) then
       write (gol,'("[ReadFromFile_MDF_r4_4d]  Variable """,a,""" in file """,a,""":")') trim(vname),trim(fname);call goPr
       write (gol,'("[ReadFromFile_MDF_r4_4d]    Was expecting """,a,""" type, but found """,a,""" type.")') trim(MDF_DATATYPE_NAME(MDF_FLOAT)),trim(MDF_DATATYPE_NAME(xtype));call goPr
       if(.not. present(lenient_typing)) then
          write (gol,'("[ReadFromFile_MDF_r4_4d]    Aborting.")');call goErr
          status=1;return
       else 
          write (gol,'("[ReadFromFile_MDF_r4_4d]    Using default type promotion.")'); call goPr
       endif
    endif

    call MDF_Get_Var(hid,vid,buffer,status)
    IF_NOTOK_RETURN(status=1)

    call MDF_Close(hid,status)
    IF_NOTOK_RETURN(status=1)

    status = 0

  end subroutine ReadFromFile_MDF_r4_4d

  subroutine ReadFromFile_MDF_r4_5d(fname,ftype,vname,buffer,lenient_typing,status)

    character(len=*), intent(in)	:: fname  ! file name
    integer, intent(in)			:: ftype  ! MDF_HDF4, MDF_NETCDF, MDF_NETCDF4, MDF_HDF5
    character(len=*), intent(in)	:: vname  ! variable name
    real(4), intent(out)		:: buffer(:,:,:,:,:)
    logical, optional, intent(in)	:: lenient_typing
    integer, intent(out)		:: status

    integer				:: hid
    integer				:: vid
    integer				:: xtype
    integer				:: ndims

    character(len=*), parameter		:: rname = mname//'/ReadFromFile_MDF_r4_5d'

    call MDF_open(fname,ftype,mode=MDF_READ,hid=hid,status=status)
    IF_NOTOK_RETURN(status=1)

    call MDF_Inq_VarID(hid,vname,vid,status)
    IF_NOTOK_RETURN(status=1)

    call MDF_Inquire_Variable(hid,vid,status,xtype=xtype,ndims=ndims)
    IF_NOTOK_RETURN(status=1)
    
    if(ndims /= 5) then
       write (gol,'("[ReadFromFile_MDF_r4_5d]  Variable """,a,""" in file """,a,""":")') trim(vname),trim(fname);call goPr
       write (gol,'("[ReadFromFile_MDF_r4_5d]    Was expecting 5 dimensions but found ",i,".")') ndims;call goPr
       write (gol,'("[ReadFromFile_MDF_r4_5d]    Aborting.")');call goErr
       status=1;return
    endif

    if(xtype /= MDF_FLOAT) then
       write (gol,'("[ReadFromFile_MDF_r4_5d]  Variable """,a,""" in file """,a,""":")') trim(vname),trim(fname);call goPr
       write (gol,'("[ReadFromFile_MDF_r4_5d]    Was expecting """,a,""" type, but found """,a,""" type.")') trim(MDF_DATATYPE_NAME(MDF_FLOAT)),trim(MDF_DATATYPE_NAME(xtype));call goPr
       if(.not. present(lenient_typing)) then
          write (gol,'("[ReadFromFile_MDF_r4_5d]    Aborting.")');call goErr
          status=1;return
       else 
          write (gol,'("[ReadFromFile_MDF_r4_5d]    Using default type promotion.")'); call goPr
       endif
    endif

    call MDF_Get_Var(hid,vid,buffer,status)
    IF_NOTOK_RETURN(status=1)

    call MDF_Close(hid,status)
    IF_NOTOK_RETURN(status=1)

    status = 0

  end subroutine ReadFromFile_MDF_r4_5d
end module ReadFromFile_MDF
