! Time-stamp: <fe8.zeus.fairmont.rdhpcs.noaa.gov:/home/Andy.Jacobson/TM5/proj/ctemis/branches/ensemble/src/emission_co2__ensemble.F90: 09 Jul 2015 (Thu) 23:49:30 UTC>

! TODO:
!  MPI-proof this code by wrapping it in myid .eq root conditionals

#define TRACEBACK write (gol,'("in ",a," (",a,", line",i5,")")') rname, __FILE__, __LINE__; call goErr
#define IF_NOTOK_RETURN(action) if (status/=0) then; TRACEBACK; action; return; end if
#define IF_ERROR_RETURN(action) if (status> 0) then; TRACEBACK; action; return; end if
#include "tm5.inc"
  
module emission_co2

  use chem_param,		only : xmco2, xmc, ntrace_emis
  use global_types,		only : emis_data
  use emission_data,		only : do_add_2d
  use emission_common,		only : co2_bio
  use emission_common,		only : co2_ocn
  use emission_common,		only : co2_ff
  use emission_common,		only : co2_fires
  use emission_common,		only : nmembersloc
  use emission_common,		only : parms,region_map
  use dims,			only : nlon360, nlat180
  use go,			only : gol, goErr, goPr

  implicit none

  private

  public		:: emission_co2_init, emission_co2_calc, emission_co2_apply, emission_co2_done
  public		:: emis_co2

  character(len=*), parameter			:: mname = 'emission_co2'
  
  type(emis_data),dimension(:,:),allocatable	:: emis_co2

  character(len=1024)				:: ParmsInputDir
  character(len=2048)				:: ParmsMapNc
  character(len=10)				:: ParmsCDate
  integer					:: itau_parms = -999
  integer					:: current_ilag
  real*4,dimension(:,:,:), allocatable		:: raw_parameters_mean
  real*4,dimension(:,:,:,:), allocatable		:: raw_parameters_deviations
  integer					:: itau_lastcalc = -999
 
contains

  subroutine emission_co2_init(status)

    use dims,			only : im, jm, nregions, nlon360, nlat180, idatei, idate
    use emission_co2_bio,	only : emission_co2_bio_init
    use emission_co2_fires,	only : emission_co2_fires_init
    use emission_co2_ff,	only : emission_co2_ff_init
    use emission_co2_ocean,	only : emission_co2_ocean_init
    use emission_common,	only : EmisInputDir
    use GO,			only : TrcFile, Init, Done, ReadRc
    use global_data,		only : rcfile
    use ReadFromFile_MDF,	only : ReadFromFile
    use MDF,			only : MDF_NETCDF
    use ParTools,		only : myid, root

    implicit none

    character(len=*), parameter	:: rname = mname//'/emission_co2_init'
    integer			:: region, imr, jmr, itrace
    integer, intent(out)	:: status
    type(TrcFile)		:: rcF

    call Init( rcF, rcfile, status )
    IF_NOTOK_RETURN(status=1)

    call ReadRc( rcF, 'emis.input.dir', EmisInputDir ,status ,default='./')
    IF_ERROR_RETURN(status=1)

    call ReadRc( rcF, 'parameters.input.dir', ParmsInputDir ,status ,default='./')
    IF_ERROR_RETURN(status=1)

    call ReadRc( rcF, 'parameters.input.cdate', ParmsCDate ,status ,default='0000-00-00')
    IF_ERROR_RETURN(status=1)

    call ReadRc( rcF, 'parameters.map.nc', ParmsMapNc ,status, default= '/scratch2/portfolios/BMC/co2/tools/shared/aux/iregions.nc')
    IF_ERROR_RETURN(status=1)

    call Done( rcF, status )
    IF_NOTOK_RETURN(status=1)

    write (gol,'("[emission_co2_init]	default input dir for emissions files is """,a,""".")') trim(EmisInputDir); call goPr

    allocate(emis_co2(nregions,nmembersloc))
    do region=1,nregions
       imr=im(region)
       jmr=jm(region)
       do itrace=1,nmembersloc
          allocate(emis_co2(region,itrace)%surf(imr,jmr))
!          write (gol,'("[emission_co2_init]		reg ",i1,", itrace ",i1,": ",i3," x ",i3,".")') region,itrace,imr,jmr; call goPr
       end do
    enddo

    allocate(parms(nmembersloc))

    allocate(co2_bio(nmembersloc))
    allocate(co2_fires(nmembersloc))
    allocate(co2_ff(nmembersloc))
    allocate(co2_ocn(nmembersloc))

    ! Allow each module to read the RC file, and set up
    ! local structures as needed. 
    call emission_co2_bio_init(status)
    IF_NOTOK_RETURN(status=1)
    call emission_co2_fires_init(status)
    IF_NOTOK_RETURN(status=1)
    call emission_co2_ff_init(status)
    IF_NOTOK_RETURN(status=1)
    call emission_co2_ocean_init(status)
    IF_NOTOK_RETURN(status=1)

  end subroutine emission_co2_init

  subroutine emission_co2_calc(status)

    use dims,			only : nlon360, nlat180, im,jm, sec_month, newsrun, idate, newday, nregions, itau, nread, itaui, idatei
    use toolbox,		only : coarsen_emission_2d
    use ParTools,		only : ntracetloc, myid, ntracet_ar, root
    use chem_param,		only : xmc
    use partools,		only : myid,root_t
    use emission_common,	only : field_1x1
    use emission_co2_bio,	only : emission_co2_bio_calc
    use emission_co2_fires,	only : emission_co2_fires_calc
    use emission_co2_ff,	only : emission_co2_ff_calc
    use emission_co2_ocean,	only : emission_co2_ocean_calc
    use ReadFromFile_MDF,	only : ReadFromFile
    use MDF,			only : MDF_Open,MDF_Inq_DimID,MDF_Inquire_Dimension,MDF_NETCDF4,MDF_Close,MDF_READ,MDF_NETCDF,MDF_Inquire
    use go_string,		only : goNum2str
    use dims,	 		only : itau
    use datetime,		only : tau2date, date2tau

    implicit none

    character(len=*), parameter	:: rname = mname//'/emission_co2_calc'

    integer			:: region, imr, jmr, n, mem
    integer, parameter		:: add_field=0
    integer, intent(out)	:: status
    logical			:: flux_updated,any_flux_updated,parms_updated
    integer			:: imember
    type(field_1x1)		:: co2_totalflux
    integer			:: ilag,itrace
    integer			:: offsetn=0
    real,dimension(240)		:: these_parms
    character(len=10)		:: cdate
    character(len=2048)		:: parmfile
    logical			:: file_found
    integer,dimension(6)	:: idate_parms
    integer			:: dimid,hid,nweeks,nparms,nmembers
    integer			:: ndim,idim
    character(len=64)		:: dimname

    if(itau_lastcalc .eq. -999) then
       itau_lastcalc = itau - 86400 ! initialize to any value which is not *now*
    endif

    if(itau == itau_lastcalc) return

    itau_lastcalc = itau

    if(myid > 0) offsetn=sum(ntracet_ar(0:myid-1))

    ! need to read the parameters (once) here, because idatei is not yet set when emission_co2_init is called.
    ! all nodes read raw parameters

    if(.not. allocated(region_map)) then ! this is our one-time signal

       allocate(region_map(nlon360,nlat180))

       call ReadFromFile(fname=trim(ParmsMapNc),ftype=MDF_NETCDF,vname='regions',buffer=region_map,status=status)
       IF_NOTOK_RETURN(status=1)

       ! read parameters
       !
       ! assume that there is a netCDF file with nlag's-worth of weekly parameters,
       ! starting at this run's start time and named using that date.
       
       cdate=trim(goNum2str(idatei(1),'(i4.4)'))//'-'//trim(goNum2str(idatei(2),'(i2.2)'))//'-'//trim(goNum2str(idatei(3),'(i2.2)'))
       parmfile=trim(ParmsInputDir)//'/scaling.factor.240.'//trim(cdate)//'.nc'
       itau_parms=itaui
       
       inquire(file=trim(parmfile),exist=file_found)

       if(.not. file_found) then
          write (gol,'("[emission_co2_calc] Cannot find parameters file with start date.")'); call goPr
          write (gol,'("[emission_co2_calc] Expected to find """,a,""".")') trim(parmfile);call goPr
          status=1
          IF_NOTOK_RETURN(status=1)
       end if

       ! init MDF here
       call MDF_Open(filename=parmfile,ftype=MDF_NETCDF4,mode=MDF_READ, &
            hid=hid,status=status)
       IF_NOTOK_RETURN(status=1)

       ! get length of dimension named either nweeks or nlag 
       nweeks = -1
       
       ! number of dimensions
       call MDF_Inquire(hid=hid, status=status, ndimensions=ndim)
       IF_NOTOK_RETURN(status=1)
       
       ! loop over dimensions
       do idim = 1, ndim
          ! get dimension name
          call MDF_Inquire_Dimension(hid, idim, status, name=dimname)
          IF_NOTOK_RETURN(status=1)
          ! check to see if dim name is 'nweeks' or 'nlag'
          if (('nweeks' == trim(dimname)) .or. ('nlag' == trim(dimname))) then
             call MDF_Inquire_Dimension(hid=hid,dimid=idim,status=status,length=nweeks)
             IF_NOTOK_RETURN(status=1)
             ! leave the idim loop
             exit
          end if
       end do  ! dimension loop
          
       if (nweeks .eq. -1) then
          write (gol,'("no dimension `nweeks` or `nlag` found in file : ",a)') trim(parmfile); call goErr
          TRACEBACK; status=1; return
       endif

       call MDF_Inq_DimID(hid=hid,name="parameters",dimid=dimid,status=status)
       IF_NOTOK_RETURN(status=1)
       
       call MDF_Inquire_Dimension(hid=hid,dimid=dimid,status=status,length=nparms)
       IF_NOTOK_RETURN(status=1)
       
       call MDF_Inq_DimID(hid=hid,name="nmembers",dimid=dimid,status=status)
       IF_NOTOK_RETURN(status=1)
       
       call MDF_Inquire_Dimension(hid=hid,dimid=dimid,status=status,length=nmembers)
       IF_NOTOK_RETURN(status=1)
       
       call MDF_Close(hid=hid,status=status)
       IF_NOTOK_RETURN(status=1)
       
       allocate(raw_parameters_mean(nparms,nweeks,1)) ! 1 is date dimension
       allocate(raw_parameters_deviations(nparms,nmembers,nweeks,1)) ! 1 is date dimension
       
       write (gol,'("[emission_co2_calc]	Reading parameters from """,a,""".")') trim(parmfile); call goPr
       current_ilag = 0 ! will force a call to parms_to_map

       ! Scaling factors are written as a 3-D variable with dimensions of nlag, nparam, nweeks.
       ! For weekly files as expected here, the dimension of nweeks is 1.
       call ReadFromFile(fname=trim(parmfile),ftype=MDF_NETCDF,vname="scaling_factor",buffer=raw_parameters_mean,status=status)
       IF_NOTOK_RETURN(status=1)
       
       call ReadFromFile(fname=trim(parmfile),ftype=MDF_NETCDF,vname="scaling_factor_deviations",buffer=raw_parameters_deviations,status=status)
       IF_NOTOK_RETURN(status=1)

    end if

    ! set parameters based on number of weeks since beginning of this run

    ilag=1+(itau-itau_parms)/(7*86400)
    if(ilag .ne. current_ilag) then
       write (gol,'("[emission_co2_calc]	Applying parameters for lag/week ",i2,".")') ilag;call goPr
       do imember = 1,nmembersloc
          these_parms = raw_parameters_mean(:,ilag,1) + raw_parameters_deviations(:,offsetn+imember,ilag,1)
          write (gol,'("[emission_co2_calc]	member ",i3," sum(parms)=",f12.4,".")') imember,sum(these_parms);call goPr
          
          call parms_to_map(these_parms,region_map,parms(imember)%surf,status=status)
       end do
       current_ilag = ilag
       IF_NOTOK_RETURN(status=1)

       parms_updated = .true.

    end if

    ! fossil fuel emissions
    call emission_co2_ff_calc(parms,co2_ff,flux_updated,parms_updated,status) ! returns FF flux in kgC 1x1_gridbox-1 month-1
    IF_NOTOK_RETURN(status=1)
    
    any_flux_updated = flux_updated

    ! terrestrial biosphere
    call emission_co2_bio_calc(parms,co2_bio,flux_updated,parms_updated,status) ! returns terrestrial biosphere flux in kgC 1x1_gridbox-1 month-1
    IF_NOTOK_RETURN(status=1)

    any_flux_updated = any_flux_updated .or. flux_updated

    ! wildfires
    call emission_co2_fires_calc(parms,co2_fires,flux_updated,parms_updated,status)
    IF_NOTOK_RETURN(status=1)

    any_flux_updated = any_flux_updated .or. flux_updated

    ! ocean emissions
    call emission_co2_ocean_calc(parms,co2_ocn,flux_updated,parms_updated,status)
    IF_NOTOK_RETURN(status=1)

    any_flux_updated = any_flux_updated .or. flux_updated

    if(any_flux_updated) then

       do imember = 1, nmembersloc
          co2_totalflux%surf = co2_ff(imember)%surf + &
               co2_bio(imember)%surf + &
               co2_fires(imember)%surf + &
               co2_ocn(imember)%surf

!          write (gol,'("[emission_co2_calc]	member ",i3," calling coarsen_e_2d with sum(co2_totalflux)=",g16.2,".")') imember,sum(co2_totalflux%surf);call goPr

          call coarsen_emission_2d('ens',360,180,co2_totalflux%surf,emis_co2(:,imember),add_field)
          IF_NOTOK_RETURN(status=1)

       end do

    end if

    status = 0

  end subroutine emission_co2_calc

  subroutine emission_co2_apply(region,status)

    use ParTools,		only : ntracetloc, myid, ntracet_ar, root
    use dims,			only : itaur, itaui
    use dims,			only : itaur, itaui, ndyn, ndyn_max
    use dims,			only : sec_month, dxy11, nlat180
    use chem_param,		only : xmc

    implicit none

    character(len=*), parameter	:: rname = mname//'/emission_co2_apply'

    real*8                      :: weight
    integer, intent(in)		:: region
    integer, intent(out)	:: status
    integer			:: itrace, offsetn=0, j
    integer			:: imember=1
    real*8			:: convfac

    if(myid > 0) offsetn=sum(ntracet_ar(0:myid-1))

    call emission_co2_calc(status)
    IF_NOTOK_RETURN(status=1)

    do imember=1,nmembersloc
       call do_add_2d(region,offsetn+imember,1,emis_co2(region,imember)%surf,xmco2,xmc)
!       write (gol,'("[emission_co2_apply]	member, region ",2i3," calling do_add_2d with sum(emis_co2)=",g18.2,".")') imember,region,sum(emis_co2(region,imember)%surf);call goPr
    enddo

    status=0

  end subroutine emission_co2_apply

  subroutine emission_co2_done(status)

    use dims,			only : nregions
    use chem_param,		only : ntrace_emis
    use emission_co2_bio,	only : emission_co2_bio_done
    use emission_co2_fires,	only : emission_co2_fires_done
    use emission_co2_ff,	only : emission_co2_ff_done
    use emission_co2_ocean,	only : emission_co2_ocean_done

    implicit none

    character(len=*), parameter	:: rname = mname//'/emission_co2_done'

    integer			:: region, itrace, imember
    integer, intent(out)	:: status


    do region=1,nregions
       do imember=1,nmembersloc
          deallocate(emis_co2(region,imember)%surf)
       end do
    enddo
    deallocate(emis_co2)

    deallocate(parms)

    deallocate(region_map)
    deallocate(raw_parameters_mean)
    deallocate(raw_parameters_deviations)

    deallocate(co2_bio)
    deallocate(co2_fires)
    deallocate(co2_ff)
    deallocate(co2_ocn)

    call emission_co2_bio_done(status)
    IF_NOTOK_RETURN(status=1)
    call emission_co2_fires_done(status)
    IF_NOTOK_RETURN(status=1)
    call emission_co2_ff_done(status)
    IF_NOTOK_RETURN(status=1)
    call emission_co2_ocean_done(status)
    IF_NOTOK_RETURN(status=1)

  end subroutine emission_co2_done


  subroutine parms_to_map(inbuf,region_map,parms,status)

    implicit none

    character(len=*), parameter		:: rname = mname//'/parms_to_map'
    integer				:: ireg
    integer, intent(out)		:: status
    real*8,dimension(:)			:: inbuf
    integer,dimension(:,:),intent(in)	:: region_map
    real,dimension(:,:),intent(out)	:: parms

    do ireg = 1,240 !min(inbuf),max(inbuf)
       where (region_map .eq. ireg) parms = inbuf(ireg)
    end do

    status=0
  end subroutine parms_to_map

end module emission_co2
