! Time-stamp: <tfe02:/home/Andy.Jacobson/TM5/proj/ctemis/branches/bio_dogfood/src/emission_co2_bio__dogfood.F90: 20 Sep 2015 (Sun) 00:05:05 UTC>

! todo:

#define TRACEBACK write (gol,'("in ",a," (",a,", line",i5,")")') rname, __FILE__, __LINE__; call goErr
#define IF_NOTOK_RETURN(action) if (status/=0) then; TRACEBACK; action; return; end if
#define IF_ERROR_RETURN(action) if (status> 0) then; TRACEBACK; action; return; end if
#include "tm5.inc"

! Read in and apply pre-computed terrestrial biosphere emissions of CO2 in mol m-2 s-1.  Positive is source to atmosphere.

module emission_co2_bio
  
  use GO,		only : gol, goErr, goPr, goLabel
  use Dims,		only : nlon360,nlat180
  use emission_common,	only : enkf_parms, field_1x1, EmisInputDir, co2_bio
  use emission_common,	only : nmembersloc

  implicit none

  character(len=*), parameter	:: mname = 'emission_co2_bio'

  private

  public	:: emission_co2_bio_init, emission_co2_bio_done, emission_co2_bio_calc

  character(len=1024)			:: BioInputDir
  character(len=100)			:: fname_prefix
  integer				:: islot_calc
  real,dimension(nlon360,nlat180,16)	:: slot_emis

contains

  subroutine emission_co2_bio_init(status)

    use GO,		only : TrcFile, Init, Done, ReadRc
    use global_data,	only : rcfile

    implicit none

    character(len=*), parameter	:: rname = mname//'/emission_co2_bio_init'

    integer, intent(out)	:: status
    type(TrcFile)		:: rcF

    islot_calc = -1 ! signals no emissions loaded

    call Init( rcF, rcfile, status )
    IF_NOTOK_RETURN(status=1)

    call ReadRc( rcF, 'bio.input.dir', BioInputDir ,status ,default=EmisInputDir)
    IF_ERROR_RETURN(status=1)

    call ReadRc( rcF, 'bio.fname.prefix', fname_prefix ,status ,default=EmisInputDir)
    IF_ERROR_RETURN(status=1)

    call Done( rcF, status )
    IF_NOTOK_RETURN(status=1)

    write (gol,'("[emission_co2_bio_init]	terrestrial bio input files to be read from """,a,""".")') trim(BioInputDir); call goPr
    write (gol,'("[emission_co2_bio_init]	          with filename of form ",a,".YYYYMMDD.nc.")') trim(fname_prefix); call goPr

  end subroutine emission_co2_bio_init

  subroutine emission_co2_bio_calc(parms,co2_bio,parms_updated,flux_updated,status)

    use dims,			only : sec_month, newsrun, newday, idate, itau, itaui, sec_year
    use ReadFromFile_MDF,	only : ReadFromFile
    use MDF,			only : MDF_NETCDF
    use emission_data,		only : flux_to_gridbox_per_month
    use chem_param,		only : xmc
    use go_string,		only : goNum2str

    implicit none
    
    character(len=*), parameter			:: rname = mname//'/emission_co2_bio_calc'
    
    type(enkf_parms), dimension(:), intent(in)	:: parms
    type(field_1x1), dimension(:), intent(out)	:: co2_bio
    integer, intent(out)			:: status
    logical, intent(in)				:: parms_updated
    logical, intent(out)			:: flux_updated

    integer					:: imember, islot_now
    character(len=2048)				:: ncname
    real					:: month2year

    flux_updated = .false.

    if (newsrun .or. newday) then

      flux_updated = .true.

      ncname=trim(BioInputDir)//'/'//trim(fname_prefix)//'.'//trim(goNum2Str(idate(1),'(i4.4)'))//trim(goNum2Str(idate(2),'(i2.2)'))//trim(goNum2Str(idate(3),'(i2.2)'))//'.nc'
      write(gol,'(a)') '[emission_co2_bio_calc] '//trim(goNum2Str(idate(1),'(i4.4)'))//'/'//trim(goNum2Str(idate(2),'(i2.2)'))//'/'//trim(goNum2Str(idate(3),'(i2.2)'))//' '//trim(goNum2Str(idate(4),'(i2.2)'))//':'//trim(goNum2Str(idate(5),'(i2.2)'))//':'//trim(goNum2Str(idate(6),'(i2.2)'))//' reading NEE '//' from '//trim(ncname)//'.';call goPr

      call ReadFromFile(fname=ncname,ftype=MDF_NETCDF,vname='NEE',buffer=slot_emis(:,:,:),lenient_typing=.TRUE.,status=status)
      IF_NOTOK_RETURN(status=1)
      slot_emis(:,:,:)=slot_emis(:,:,:)*(xmc/1.e3) ! from mol m-2 s-1 to kgC m-2 s-1

    endif

    islot_now = 1 + (idate(4)*3600 + idate(5)*60 + idate(6))/5400  ! integer division, ignores remainders

    if (flux_updated .or. parms_updated .or. (islot_calc .ne. islot_now))  then

       write (gol,'("[emission_co2_bio_calc]	At ",i2.2,":",i2.2,":",i2.2,", islot is ",i2,".")') &
            idate(4),idate(5),idate(6),islot_now
       call goPr

       flux_updated = .true.
       islot_calc = islot_now

       ! month2year only used for diagnostic output
       month2year = sec_year/sec_month

       do imember = 1,nmembersloc ! forward has nmembersloc = 1

          ! slot_emis is in kgC m-2 s-1
          co2_bio(imember)%surf = slot_emis(:,:,islot_now)*parms(imember)%surf ! apply parameters
!          co2_bio(imember)%surf = slot_emis(:,:,islot_now) ! fake with 1.0 parameter map
          
          call flux_to_gridbox_per_month(co2_bio(imember)%surf) ! convert from kgC m-2 s-1 to kgC 1x1_gridbox-1 month-1
          
          write (gol,'("[emission_co2_bio_calc]	member ",i3,": terrestrial biosphere emission global integral ",f12.4," PgC/yr.")') &
               imember,sum(co2_bio(imember)%surf)*1.0e-12*month2year
          call goPr
          
       end do

    endif

  end subroutine emission_co2_bio_calc

  subroutine emission_co2_bio_done(status)

    implicit none

    integer, intent(out)			:: status

    character(len=*), parameter	:: rname = mname//'/emission_co2_bio_done'

    status=0
    return

  end subroutine emission_co2_bio_done

end module emission_co2_bio
