! Time-stamp: <tfe09:/home/Andy.Jacobson/TM5/proj/ctemis/branches/fires_GFED4/src/emission_co2_fires__GFED4.F90: 10 Dec 2015 (Thu) 04:01:41 UTC>

#define TRACEBACK write (gol,'("in ",a," (",a,", line",i5,")")') rname, __FILE__, __LINE__; call goErr
#define IF_NOTOK_RETURN(action) if (status/=0) then; TRACEBACK; action; return; end if
#define IF_ERROR_RETURN(action) if (status> 0) then; TRACEBACK; action; return; end if
#include "tm5.inc"

! Compute wildfire emissions of CO2.  Positive is source to atmosphere.

module emission_co2_fires

  use GO,		only : gol, goErr, goPr, goLabel
  use Dims,		only : nregions, nlon360, nlat180
  use emission_common,	only : enkf_parms, field_1x1, EmisInputDir, co2_fires
  use emission_common,	only : nmembersloc
  use ReadFromFile_MDF,	only : ReadFromFile
  use MDF,		only : MDF_NETCDF

  implicit none

  character(len=*), parameter			:: mname = 'emission_co2_fires'

  private
  character(len=2048)				:: fires_prefix
  character(len=256)				:: fires_varname
  character(len=1024)				:: FiresInputDir
  real*4,dimension(360,180,12)			:: fires_read
  integer, dimension(12)			:: spm ! seconds per month (from netCDF input)

  logical					:: factors_verbose

  logical					:: factors_daily
  character(len=1024)				:: factors_daily_dir
  character(len=64)				:: factors_daily_prefix
  real*4, dimension(:,:), allocatable		:: factors_daily_values

  logical					:: factors_3hrly
  character(len=1024)           		:: factors_3hrly_dir
  character(len=64)				:: factors_3hrly_prefix
  real*4, dimension(:,:,:), allocatable		:: factors_3hrly_values

  real*4, dimension(nlon360,nlat180)		:: factors_values

  integer					:: islot_last

  public					:: emission_co2_fires_init
  public					:: emission_co2_fires_done
  public					:: emission_co2_fires_calc

  
contains

  subroutine emission_co2_fires_init(status)

    use GO,		only: TrcFile, Init, Done, ReadRc
    use global_data,	only: rcfile
    use dims,		only: im,jm,nregions

    implicit none

    character(len=*), parameter	:: rname = mname//'/emission_co2_fires_init'

    integer			:: region, imr, jmr, n
    integer, intent(out)	:: status
    type(TrcFile)		:: rcF

    call Init( rcF, rcfile, status )
    IF_NOTOK_RETURN(status=1)
    call ReadRc( rcF, 'fires.input.dir', FiresInputDir ,status ,default=EmisInputDir)
    IF_ERROR_RETURN(status=1)
    call ReadRc( rcF, 'fires.fname.prefix', fires_prefix,status ,default=EmisInputDir)
    IF_ERROR_RETURN(status=1)

    call ReadRc( rcF, 'fire.factors.verbose', factors_verbose , status, .true.)
    IF_ERROR_RETURN(status=1)

    write (gol,'("[emission_co2_fires_init]	fires input files to be read from """,a,""".")') trim(FiresInputDir)
    call goPr
    write (gol,'("[emission_co2_fires_init]	   from files with names like ",a,".YYYYMMDD.nc.")') trim(fires_prefix)
    call goPr

    call ReadRc( rcF, 'fire.factors.daily', factors_daily , status, .false.)
    IF_ERROR_RETURN(status=1)
    
    if(factors_daily) then

       call ReadRc( rcF, 'fire.factors.daily.dir', factors_daily_dir, status, &
            "/scratch1/portfolios/BMC/co2/GFED-CASA/2015/GFED4.1s/temporal_scaling_factors")
       IF_ERROR_RETURN(status=1)

       call ReadRc( rcF, 'fire.factors.daily.prefix', factors_daily_prefix, status, &
            "GFED4.1s.daily_sf")
       IF_ERROR_RETURN(status=1)

       allocate(factors_daily_values(nlon360,nlat180))

       write (gol,'("[emission_co2_fires_init]	   daily factors from files like ",a,"/",a,"YYYYMMDD.nc.")') trim(factors_daily_dir),trim(factors_daily_prefix)
       call goPr
       
    endif

    call ReadRc( rcF, 'fire.factors.3hrly', factors_3hrly , status, .false.)
    IF_ERROR_RETURN(status=1)

    if(factors_3hrly .and. .not. factors_daily) then
       write (gol,'("[emission_co2_fires_init]	ERROR: cannot use 3-hourly factors without also using daily factors.")')
       status=1
       call goErr
       return
    endif

    if(factors_3hrly) then

       call ReadRc( rcF, 'fire.factors.3hrly.dir', factors_3hrly_dir, status, &
            "/scratch1/portfolios/BMC/co2/GFED-CASA/2015/GFED4.1s/temporal_scaling_factors")
       IF_ERROR_RETURN(status=1)

       call ReadRc( rcF, 'fire.factors.3hrly.prefix', factors_3hrly_prefix, status, &
            "GFED4.1s.3hrly_sf")
       IF_ERROR_RETURN(status=1)

       allocate(factors_3hrly_values(nlon360,nlat180,8))

       write (gol,'("[emission_co2_fires_init]	   3-hourly factors from files like ",a,"/",a,"YYYYMM.nc.")') trim(factors_3hrly_dir),trim(factors_3hrly_prefix)
       call goPr

       islot_last = -1 ! signal that factor needs to be set
    endif

    call Done( rcF, status )
    IF_NOTOK_RETURN(status=1)


  end subroutine emission_co2_fires_init

  subroutine emission_co2_fires_calc(parms,co2_fires,flux_updated,parms_updated,status)

    use dims,			only : sec_month, idate, newday, sec_year, newsrun, newmonth, newyr
    use emission_data,		only : chardate, flux_to_gridbox_per_month
    use go_string,		only : goNum2str
    use chem_param,		only : xmc

    implicit none

    character(len=*), parameter			:: rname = mname//'/emission_co2_fires_calc'

    ! parms actually not used here since we do not optimize FIRES emissions
    type(enkf_parms), dimension(:), intent(in)	:: parms

    type(field_1x1), dimension(:), intent(out)	:: co2_fires
    integer, intent(out)			:: status
    logical, intent(out)			:: flux_updated
    logical, intent(in)				:: parms_updated

    integer					:: imember,islot,imon
    real					:: month2year
    character(len=10)				:: cdate


    flux_updated = .false.

    factors_values = 1.
    
    if(newyr .or. newsrun) then

       flux_updated = .true.

       cdate=trim(goNum2str(idate(1),'(i4.4)'))

       ! units of biomass burning are gC m-2 s-1
       call ReadFromFile(fname=trim(FiresInputDir)//'/'//trim(fires_prefix)//'_'//trim(cdate)//'.nc', &
            ftype=MDF_NETCDF,vname='BB',buffer=fires_read,lenient_typing=.TRUE.,status=status)
       IF_NOTOK_RETURN(status=1)

       if(factors_daily .or. factors_3hrly) then
          ! get seconds-per-month, spm
          call ReadFromFile(fname=trim(FiresInputDir)//'/'//trim(fires_prefix)//'_'//trim(cdate)//'.nc', &
               ftype=MDF_NETCDF,vname='spm',buffer=spm,lenient_typing=.TRUE.,status=status)
          IF_NOTOK_RETURN(status=1)

          do imon=1,12
             ! change units of fires_read to gC m-2 month-1 (for application of factors)
             fires_read(:,:,imon)=fires_read(:,:,imon)*real(spm(imon))
          enddo

       endif

    end if

    if(factors_daily) then

       if(newsrun .or. newday) then
       
          flux_updated = .true.
          
          cdate=trim(goNum2str(idate(1),'(i4.4)'))//trim(goNum2str(idate(2),'(i2.2)'))//trim(goNum2str(idate(3),'(i2.2)'))
    
          call ReadFromFile(fname = trim(factors_daily_dir)//'/'//trim(goNum2str(idate(1),'(i4.4)'))//'/'//trim(factors_daily_prefix)//trim(cdate)//'.nc', &
               ftype = MDF_NETCDF, vname = 'Fraction_of_Emissions', lenient_typing = .true., &
               buffer = factors_daily_values, status = status)
          IF_NOTOK_RETURN(status=1)

       endif

      factors_values=factors_values * factors_daily_values  ! implicit conversion from per-month to per-day

   endif

   if(newmonth .or. newsrun) then

      flux_updated = .true.
      
      cdate=trim(goNum2str(idate(1),'(i4.4)'))//trim(goNum2str(idate(2),'(i2.2)'))

      if(factors_3hrly) then  ! new 3hrly factors each month

          call ReadFromFile(fname = trim(factors_3hrly_dir)//'/'//trim(goNum2str(idate(1),'(i4.4)'))//'/'//trim(factors_3hrly_prefix)//trim(cdate)//'.nc', &
               ftype = MDF_NETCDF, vname = 'Fraction_of_Emissions', lenient_typing = .true., &
               buffer = factors_3hrly_values, status = status)
          IF_NOTOK_RETURN(status=1)

      endif

   endif

   if(factors_3hrly) then

      islot=1+idate(4)/3 ! integer truncating arithmetic 1+0:23/3 -> 1 1 1 2 2 2 .... 8 8 8 

      factors_values=factors_values * factors_3hrly_values(:,:,islot) ! implicit conversion from per-day to per-3hourly
      factors_values=factors_values/(3.*3600.) ! convert to per-second from per-3hourly
      
      if(islot .ne. islot_last) then
         flux_updated = .true.  ! could track former_slot
         islot_last = islot
      endif

      if(factors_verbose) then
         write (gol,'("[emission_co2_fires_calc]  diurnal factors: hour ",i2," is 3-hr slot ",i2,".")') idate(4),islot
         call goPr
      endif

   elseif (factors_daily) then ! i.e., if daily factors but not 3-hourly factors
      factors_values=factors_values/(24.*3600.) ! convert to per-second from per-daily
   endif

   if(.not. flux_updated) return

   ! used for diagnostic output only
   month2year = sec_year/sec_month

   do imember=1,nmembersloc

      ! note that fires_read right now has the following units:
      !      factors_daily   factors_3hrly     fires_read units
      !          F               F              gC m-2 s-1
      !          T               F              gC m-2 day-1
      !          T               T              gC m-2 (3-hours)-1
      ! 
      ! factors_values contains the seconds-per-day or 3*3600 (seconds per 3-hours) 
      ! needed to convert fires_read to gC m-2 s-1.

      co2_fires(imember)%surf=fires_read(:,:,idate(2))*factors_values/1.0e3 ! from fires_read units gC m-2 (some time interval)-1 to kgC m-2 s-1
      
      call flux_to_gridbox_per_month(co2_fires(imember)%surf) ! convert from kgC m-2 s-1 to kgC 1x1_gridbox-1 month-1
      
      write (gol,'("[emission_co2_fires_calc]	member ",i4,": wildfire emission global integral ",f12.4," PgC/yr.")') imember,sum(co2_fires(imember)%surf)*1.0e-12*month2year
      call goPr
      
   end do
   

  end subroutine emission_co2_fires_calc

  subroutine emission_co2_fires_done(status)

    use dims,		only: nregions

    implicit none

    integer, intent(out)			:: status

    character(len=*), parameter	:: rname = mname//'/emission_co2_fires_done'

    status=0
    return

  end subroutine emission_co2_fires_done


end module emission_co2_fires

