! Time-stamp: <fe8.zeus.fairmont.rdhpcs.noaa.gov:/home/Andy.Jacobson/TM5/proj/ctemis/branches/taka2009/src/emission_co2_ocean__Taka2009.F90: 13 May 2015 (Wed) 21:17:20 UTC>

#define TRACEBACK write (gol,'("in ",a," (",a,", line",i5,")")') rname, __FILE__, __LINE__; call goErr
#define IF_NOTOK_RETURN(action) if (status/=0) then; TRACEBACK; action; return; end if
#define IF_ERROR_RETURN(action) if (status> 0) then; TRACEBACK; action; return; end if
#include "tm5.inc"

! Compute air-sea exchange of CO2 in mol m-2 s-1.  Positive is source to atmosphere.
!       This version uses the Takahashi et al. (2009) climatological pCO2 fields.
!
!   - by default, uses modeled atmospheric CO2 mole fraction to compute air-sea exchange
!      -alternatively: use GLOBALVIEW-CO2 marine boundary layer reference CO2 surface
!      -alternatively: use year-2000 pCO2 (air) from Takahashi et al. (2009)
!
!   - by default, applies assumed pCO2(sw) trend from Takahashi et al. (2009)
!      -alternatively: use no trend (appropriate for year 2000)
!

module emission_co2_ocean
  
  use GO,		only: gol, goErr, goPr, goLabel
  use Dims,		only: nregions, nlon360,nlat180
  use Meteo,		only: Set
  use emission_common,	only: enkf_parms, co2_ocn, EmisInputDir, field_1x1
  use emission_common,	only: nmembersloc

  implicit none

  character(len=*), parameter	:: mname = 'emission_co2_ocn'

  private

  public	:: emission_co2_ocean_init, emission_co2_ocean_done, emission_co2_ocean_calc

  ! use this to potentially override the EmisInputDir
  character(len=2048)		:: OceanInputDir

  ! file name 
  character(len=1024)		:: OceanPCO2File
  character(len=1024)		:: OceanSolFile
  character(len=1024)		:: OceanGridMapFile
  character(len=1024)		:: GVGridMapFile
  character(len=1024)		:: GlobalviewFile

  character(len=20)		:: Ocean_pco2_air

  logical			:: apply_sw_trend = .TRUE.

  integer,dimension(:,:),allocatable	:: grid_region
  integer,dimension(:,:),allocatable	:: grid_i
  integer,dimension(:,:),allocatable	:: grid_j

  real,dimension(nlon360,nlat180)	:: schmidtno,solubility,svp,pco2sw_in

  real,dimension(:,:),allocatable	:: gv_mbl
  real,dimension(:),allocatable		:: gv_dd
  integer,dimension(:),allocatable	:: gv_j

contains

  subroutine emission_co2_ocean_init(status)

    use GO,			only : TrcFile, Init, Done, ReadRc, goLoCase
    use global_data,		only : rcfile
    use dims,			only : im,jm,nregions, iglbsfc
    use Meteo,			only : u10m_dat, v10m_dat, ci_dat, m_dat, sp_dat
    use global_data,		only : mass_dat
    use MDF,			only : MDF_NETCDF
    use ReadFromFile_MDF,	only : ReadFromFile


    implicit none

    character(len=*), parameter ::  rname = mname//'/emission_co2_ocean_init'

    integer, intent(out)	:: status
    type(TrcFile)		:: rcF

    call Init( rcF, rcfile, status )
    IF_NOTOK_RETURN(status=1)
    call ReadRc( rcF, 'ocean.input.dir', OceanInputDir ,status, default=EmisInputDir)
    IF_ERROR_RETURN(status=1)
    call ReadRc( rcF, 'ocean.pco2.file', OceanPCO2File ,status, default="Takahashi_2009_verC.1x1.nc")
    IF_ERROR_RETURN(status=1)
    call ReadRc( rcF, 'ocean.pco2.trend.sw', apply_sw_trend ,status, default=.TRUE.)
    IF_ERROR_RETURN(status=1)
    call ReadRc( rcF, 'ocean.solubility.file', OceanSolFile ,status, default="WOA09.monthly.co2.sol_and_Sc.180.nc")
    IF_ERROR_RETURN(status=1)
    call ReadRc( rcF, 'ocean.pco2.air', Ocean_pco2_air ,status, default="interactive")
    IF_ERROR_RETURN(status=1)

    ! Even if pCO2 air condition is takahashi or globalview, we need these r,i,j triplets
    ! to get the surface pressure (to convert dry air mole fraction to partial pressure).
    call ReadRc( rcF, 'gridmap.1x1.file', OceanGridMapFile ,status, default="rij_glb3x2_nam1x1.nc")
    IF_ERROR_RETURN(status=1)
    allocate(grid_region(nlon360,nlat180))
    allocate(grid_i(nlon360,nlat180))
    allocate(grid_j(nlon360,nlat180))


    select case ( ocean_pco2_air )

    case ( 'interactive' )
       ! no op

    case ( 'globalview' )
       call ReadRc( rcF, 'gvmap.1x1.file', GVGridMapFile ,status, default="gv_j.nc")
       IF_ERROR_RETURN(status=1)
       call ReadRc( rcF, 'globalview.file', GlobalviewFile ,status, default="gv11_mbl.nc")
       IF_ERROR_RETURN(status=1)
       allocate(gv_j(nlat180))

    case ( 'takahashi' )
       ! no op

    case default
       write (gol,'("unsupported ocean.pco2.air value in rc file : """,a,"""")') trim(ocean_pco2_air);call goPr
       write (gol,'("supported strings are ""interactive"", ""globalview"", and ""takahashi""")');call goPr
       status=1
       IF_NOTOK_RETURN(status=1)

    end select

    call Done( rcF, status )
    IF_NOTOK_RETURN(status=1)

    write (gol,'("[emission_co2_ocean_init]	seawater pCO2 data to be read from """,a,"/",a,""".")') trim(OceanInputDir), trim(OceanPCO2File)
    call goPr
    if(apply_sw_trend) then
       write (gol,'("[emission_co2_ocean_init]	   seawater pCO2 will be generated from year-2000 fields plus a 1.5 uatm/yr trend.")')
    else
       write (gol,'("[emission_co2_ocean_init]	   seawater pCO2 will be generated from year-2000 fields with NO trend.")')
    end if
    call goPr

    call ReadFromFile(fname=trim(OceanInputDir)//'/'//trim(OceanGridMapFile), &
         ftype=MDF_NETCDF,vname='r', buffer=grid_region, status=status)
    IF_NOTOK_RETURN(status=1)
    
    call ReadFromFile(fname=trim(OceanInputDir)//'/'//trim(OceanGridMapFile), &
         ftype=MDF_NETCDF,vname='i', buffer=grid_i, status=status)
    IF_NOTOK_RETURN(status=1)
    
    call ReadFromFile(fname=trim(OceanInputDir)//'/'//trim(OceanGridMapFile), &
         ftype=MDF_NETCDF,vname='j', buffer=grid_j, status=status)
    IF_NOTOK_RETURN(status=1)
    
    select case ( ocean_pco2_air )

    case ( 'interactive' )
       write (gol,'("[emission_co2_ocean_init]	pCO2 (air) determined interactively from TM5 modeled mole fractions.")')
       call goPr
       write (gol,'("[emission_co2_ocean_init]	   mapping from TM5 grids to 1x1 flux grid to be read from """,a,"/",a,""".")') trim(OceanInputDir), trim(OceanGridMapFile)
       call goPr

    case ( 'globalview' )
       call ReadFromFile(fname=trim(OceanInputDir)//'/'//trim(GVGridMapFile), &
            ftype=MDF_NETCDF,vname='j', buffer=gv_j, status=status)
       IF_NOTOK_RETURN(status=1)

       write (gol,'("[emission_co2_ocean_init]	pCO2 (air) determined from GLOBALVIEW-CO2 dry air mole fraction read from """,a,"/",a,""".")') trim(OceanInputDir), trim(GlobalviewFile)
       call goPr
       write (gol,'("[emission_co2_ocean_init]	   mapping from GLOBALVIEW MBL latitudes to 1x1 flux grid to be read from """,a,"/",a,""".")') trim(OceanInputDir), trim(GVGridMapFile)
       call goPr
       write (gol,'("[emission_co2_ocean_init]	   mapping from TM5 surface pressure grids to 1x1 flux grid to be read from """,a,"/",a,""".")') trim(OceanInputDir), trim(OceanGridMapFile)
       call goPr

    case ( 'takahashi' )
       write (gol,'("[emission_co2_ocean_init]	pCO2 (air) from Takahashi year-2000 data..")')
       call goPr
       write (gol,'("[emission_co2_ocean_init]	   mapping from Takahashi 4x5 grid to 1x1 flux grid to be read from """,a,"/",a,""".")') trim(OceanInputDir), trim(OceanGridMapFile)
       call goPr

    case default
       write (gol,'("unsupported ocean.pco2.air value in rc file : ",a)') trim(ocean_pco2_air);call goPr
       write (gol,'("supported strings are ""interactive"", ""globalview"", and ""takahashi""")');call goPr
       status=1
       IF_NOTOK_RETURN(status=1)

    end select

    write (gol,'("[emission_co2_ocean_init]	   solubility and Schmidt no. to be read from """,a,"/",a,""".")') trim(OceanInputDir), trim(OceanSolFile)
    call goPr

    call Set(   u10m_dat(iglbsfc), status, used=.true. )
    IF_NOTOK_RETURN(status=1)
    call Set(   v10m_dat(iglbsfc), status, used=.true. )
    IF_NOTOK_RETURN(status=1)
    call Set(   ci_dat(iglbsfc), status, used=.true. )
    IF_NOTOK_RETURN(status=1)
    call Set(   m_dat(iglbsfc), status, used=.true. )
    IF_NOTOK_RETURN(status=1)
    
  end subroutine emission_co2_ocean_init

  subroutine emission_co2_ocean_calc(parms,co2_ocn,flux_updated,parms_updated,status)

    use Dims,			only : nlon360, nlat180, idate, itau, nread, newmonth, iglbsfc, newsrun
    use Dims,			only : sec_month,sec_year
    use emission_data,		only : flux_to_gridbox_per_month
    use MDF,			only : MDF_NETCDF
    use ReadFromFile_MDF,	only : ReadFromFile
    use Meteo,			only : u10m_dat, v10m_dat, ci_dat, m_dat, sp_dat
    use go_string,		only : goNum2str
    use chem_param,		only : xmco2, xmair, xmc, ntracet, nmembersloc
    use chem_param,		only : ico2bg, ico2ff, ico2bio, ico2ocean, ico2fires
    use datetime,		only : idate2ddate,tau2date
    use global_data,		only : mass_dat

    implicit none
    
    character(len=*), parameter			:: rname = mname//'/emission_co2_ocean_calc'

    type(field_1x1), dimension(:), intent(out)	:: co2_ocn
    type(enkf_parms), dimension(:), intent(in)	:: parms
    logical, intent(out)			:: flux_updated
    logical, intent(in)				:: parms_updated

    integer, intent(out)			:: status
    integer					:: i,j,ii,jj,imember
    real					:: month2year
    real*4,dimension(nlon360,nlat180,12)	:: buff3d
    real*4,dimension(nlon360,nlat180)		:: k92,icefrac,pco2air,pco2sw
    real,pointer,dimension(:,:)			:: u10m,v10m,ci
    integer, dimension(6)			:: idatec
    integer					:: month, gr, gi, gj, gvl, gvj
    real					:: dec_date

    flux_updated = .false.

    call tau2date(itau,idatec)
    month=idatec(2)
    dec_date=idate2ddate(idatec)

    if(trim(ocean_pco2_air) == 'globalview') then

       flux_updated = .true.

       if( .not. allocated(gv_mbl)) then
          call get_gv_mbl(trim(OceanInputDir)//'/'//trim(GlobalviewFile),status)
          IF_NOTOK_RETURN(status=1)
       end if

       ! get bracketing dates from gv_dd
       gvl = 1+floor((dec_date-1979.0)/(1./48.))

       if((gvl .lt. lbound(gv_mbl,2)) .or. (gvl .gt. ubound(gv_mbl,2))) then
          write (gol,'("Time index into gv_mbl,",i," is outside array bounds: ",2i)') gvl,lbound(gv_mbl,2),ubound(gv_mbl,2);call goPr
          status=1
          IF_NOTOK_RETURN(status=1)
       end if
    end if

    ! get necessary fields

    if(newmonth .or. newsrun) then

       flux_updated = .true.

       call ReadFromFile(fname=trim(OceanInputDir)//'/'//trim(OceanPCO2File), &
            ftype=MDF_NETCDF,vname='PCO2_SW', buffer=buff3d, status=status)
       IF_NOTOK_RETURN(status=1)

       pco2sw_in = buff3d(:,:,month)

       call ReadFromFile(fname=trim(OceanInputDir)//'/'//trim(OceanSolFile), &
            ftype=MDF_NETCDF,vname='sol', buffer=buff3d, status=status)
       IF_NOTOK_RETURN(status=1)

       solubility = buff3d(:,:,month)

       call ReadFromFile(fname=trim(OceanInputDir)//'/'//trim(OceanSolFile), &
            ftype=MDF_NETCDF,vname='Sc', buffer=buff3d, status=status)
       IF_NOTOK_RETURN(status=1)

       schmidtno = buff3d(:,:,month)

       call ReadFromFile(fname=trim(OceanInputDir)//'/'//trim(OceanSolFile), &
            ftype=MDF_NETCDF,vname='SVP', buffer=buff3d, status=status)
       IF_NOTOK_RETURN(status=1)

       svp = buff3d(:,:,month)

       if(trim(ocean_pco2_air) == 'takahashi') then
          call ReadFromFile(fname=trim(OceanInputDir)//'/'//trim(OceanSolFile), &
               ftype=MDF_NETCDF,vname='PCO2_AIR', buffer=buff3d, status=status)
          IF_NOTOK_RETURN(status=1)

          pco2air = buff3d(:,:,month)

       endif
    endif

    ! with interpolated meteo, these are different every timestep
    u10m => u10m_dat(iglbsfc)%data1(:,:,1)
    v10m => v10m_dat(iglbsfc)%data1(:,:,1)
    ci => ci_dat(iglbsfc)%data1(:,:,1)
    flux_updated = .true.

    ! following Takahashi et al. (2009), maximum impermeability of ice cover is 0.95
    icefrac = ci
    where(ci(:,:) > 0.95) icefrac=0.95

    ! make gas transfer velocity in m/s according to Wanninkhof 1992 quadratic parameterization
    k92=0.0
    where ((solubility(:,:).ge.0.0).and.(schmidtno(:,:).ge.0.0)) &     ! masked to WOA09 ocean points
         k92=(1.0-icefrac)*solubility*(0.31*(u10m**2+v10m**2)/sqrt(schmidtno/660.0))*1.0e-2/3600.0

    ! Convert pco2sw_in to pco2sw.  The _in version needs to be retained since it is
    ! only read at the beginning of each month.  Here we convert to atm from input microatm
    ! and optionally add temporal trend.
    if(apply_sw_trend) then
       pco2sw = 1.0e-6*(pco2sw_in + 1.5*(dec_date - 2000.0))  ! trend is 1.5 microatm yr-1
       !       write (gol,'("[emission_co2_ocn_calc]	at dd", f12.4,": adjusted pco2sw by ",f12.4," uatm.")') dec_date,1.5*(dec_date-2000.0)
       !       call goPr
    else
       pco2sw = 1.0e-6*pco2sw_in
    end if

    do imember = 1,nmembersloc

       ! compute pco2air
       !   need whole co2 here, so forward and inverse do different things
       !   pco2air is computed from the dry-air CO2 mole fraction.  It is then
       !   converted to CO2 partial pressure by multiplying by surface pressure,
       !   less the presumed saturation vapor pressure of water.  
       !
       !   Units:
       !     convert from Pa to atm by dividing by 101325.
       !
       ! Note that SVP is read in as p(h2o)/p(tot)
       !
       ! Use nmembersloc to figure out if this is a forward run with CO2 components,
       ! or an inverse run with an ensemble of whole-CO2 tracers.  TODO: not the
       ! best method to make this decision.

       select case ( ocean_pco2_air )
             
       case ( 'interactive' )
       
          do i=1,360
             do j=1,180
                   
                gr=grid_region(i,j)
                gi=grid_i(i,j)
                gj=grid_j(i,j)

                if(nmembersloc .eq. 1) then

                   ! forward:  total co2 = bio + ocn + fires + ff - 3*bg
                   pco2air(i,j) = sp_dat(gr)%data(gi,gj,1)*(1-svp(i,j))*(xmair/xmco2)*(1/101325.0)* &
                        (mass_dat(gr)%rm_t(gi,gj,1,ico2ff) + &
                        mass_dat(gr)%rm_t(gi,gj,1,ico2ocean) + &
                        mass_dat(gr)%rm_t(gi,gj,1,ico2bio) + &
                        mass_dat(gr)%rm_t(gi,gj,1,ico2fires) - &
                        3*mass_dat(gr)%rm_t(gi,gj,1,ico2bg)) &
                        /m_dat(gr)%data(gi,gj,1)

                else

                   ! inverse: full co2 exists in each tracer
                   pco2air(i,j) = sp_dat(gr)%data(gi,gj,1)*(1-svp(i,j))*(xmair/xmco2)*(1/101325.0)* &
                        mass_dat(gr)%rm_t(gi,gj,1,imember)/m_dat(gr)%data(gi,gj,1)
                end if
             end do
          end do
                      
       case ( 'globalview' )

          do i=1,360
             do j=1,180
                   
                gr=grid_region(i,j)
                gi=grid_i(i,j)
                gj=grid_j(i,j)
                
                gvj=gv_j(j)
                pco2air(i,j) = sp_dat(gr)%data(gi,gj,1)*(1-svp(i,j))*(1/101325.0)*1.0e-6*gv_mbl(gvj,gvl)

             end do
          end do

       case ( 'takahashi' )

          ! no op

       case default
          write (gol,'("unsupported ocean.pco2.air value in rc file : ",a)') trim(ocean_pco2_air);call goPr
          write (gol,'("supported strings are ""interactive"", ""globalview"", and ""takahashi""")');call goPr
          status=1
          IF_NOTOK_RETURN(status=1)
          
       end select

       ! make fluxes of CO2 in mol m-2 s-1
       !
       ! masking to ocean grid points only:  k92 is already WOA09-masked via
       ! solubility and Schmidt number; Takahashi pco2sw requires an additional
       ! mask

       co2_ocn(imember)%surf = 0.0
       where(pco2sw(:,:) .gt.0.0) &
          co2_ocn(imember)%surf = k92*(pco2sw-pco2air) * parms(imember)%surf    ! apply parameters by member (forward has nmembersloc=1)

       ! co2_ocn now in mol m-2 s-1

       co2_ocn(imember)%surf=co2_ocn(imember)%surf*xmc/1.0e3 ! from mol m-2 s-1 to kgC m-2 s-1

       call flux_to_gridbox_per_month(co2_ocn(imember)%surf) ! convert from kgC m-2 s-1 to kgC 1x1_gridbox-1 month-1

       ! used for diagnostic output
       month2year = sec_year/sec_month
       write (gol,'("[emission_co2_ocn_calc]	member ",i4,": oceanic emission global integral ",f12.4," PgC/yr.")') &
            imember,sum(co2_ocn(imember)%surf)*1.0e-12*month2year
       call goPr

    end do

    nullify(u10m)
    nullify(v10m)
    nullify(ci)

    status=0

  end subroutine emission_co2_ocean_calc

  subroutine get_gv_mbl(fname,status)

    use MDF,			only : MDF_open, MDF_Inq_VarID, MDF_Get_Var, MDF_Close, MDF_NETCDF, MDF_READ, MDF_Inq_DimID, MDF_Inquire_Dimension
    use dims,			only : idatei,idatee
    use datetime,		only : idate2ddate

    character(len=*),intent(in) 	:: fname
    integer,intent(out)			:: status

    integer 				:: hid, vid, dimid
    integer 				:: tstart=-1,tcount=-1,ttotal=-1,itime
    real,dimension(:),allocatable	:: ddate
    real 				:: ddatei,ddatee

    character(len=*), parameter	:: rname = mname//'/get_gv_mbl'

    ! based on start and end date of simulation, compute tstart and tcount
    
    call MDF_open(fname,ftype=MDF_NETCDF,mode=MDF_READ,hid=hid,status=status)
    IF_NOTOK_RETURN(status=1)

    call MDF_Inq_DimID(hid=hid,name='time',dimid=dimid,status=status)
    IF_NOTOK_RETURN(status=1)

    call MDF_Inquire_Dimension(hid=hid,dimid=dimid,status=status,length=ttotal)
    IF_NOTOK_RETURN(status=1)

    allocate(ddate(ttotal))
    
    call MDF_Inq_VarID(hid,name='decimal_date',varid=vid,status=status)
    IF_NOTOK_RETURN(status=1)
    
    call MDF_Get_Var(hid,vid,ddate,status)
    IF_NOTOK_RETURN(status=1)

    !
    ddatee=idate2ddate(idatee)
    ddatei=idate2ddate(idatei)

    do itime=ttotal,1,-1
       if(ddate(itime) .le. ddatei) then
          tstart=itime
          exit
       end if
    end do

    do itime=1,ttotal
       if(ddate(itime) .ge. ddatee) then
          tcount=itime-tstart+1
          exit
       end if
    end do
    
    if((tstart .lt. 1) .or. (tstart .ge. ttotal) .or. (tcount .lt. 1) .or. (tcount .gt. ttotal)) then
       write (gol,'("Cannot find appropriate time interval in GLOBALVIEW file bracketing the simulation start and end times.")');call goPr
       write (gol,'("  Simulation start/end decimal dates:",2f12.6)') ddatei,ddatee;call goPr
       write (gol,'("  GLOBALVIEW start/end decimal dates:",2f12.6)') ddate(1),ddate(ttotal);call goPr
       write (gol,'("  tstart, tcount, ttotal:",3i5)') tstart,tcount,ttotal;call goPr
       status=1
       IF_NOTOK_RETURN(status=1)
    end if

    allocate(gv_mbl(41,tstart:(tstart+tcount-1)))
    allocate(gv_dd(tcount))

!    write (gol,'("Reading from """,a,""".")') fname;call goPr
!    write (gol,'("  Simulation start/end decimal dates:",2f12.6)') ddatei,ddatee;call goPr
!    write (gol,'("  GLOBALVIEW start/end decimal dates:",2f12.6)') ddate(1),ddate(ttotal);call goPr

    call MDF_Inq_VarID(hid=hid,name='mbl',varid=vid,status=status)
    IF_NOTOK_RETURN(status=1)

    call MDF_Get_Var(hid=hid,varid=vid,values=gv_mbl,start=(/1,tstart/),count=(/41,tcount/),status=status)
    IF_NOTOK_RETURN(status=1)
    
    call MDF_Close(hid,status)
    IF_NOTOK_RETURN(status=1)
    
    deallocate(ddate)
    status = 0
    
  end subroutine get_gv_mbl

  subroutine emission_co2_ocean_done(status)

    use dims,		only: nregions

    implicit none

    integer, intent(out)			:: status

    character(len=*), parameter	:: rname = mname//'/emission_co2_ocean_done'

    if(allocated(grid_region)) then
       deallocate(grid_region)
    end if

    if(allocated(grid_i)) then
       deallocate(grid_i)
    end if

    if(allocated(grid_j)) then
       deallocate(grid_j)
    end if

    if(allocated(gv_mbl)) then
       deallocate(gv_mbl)
    end if

    if(allocated(gv_dd)) then
       deallocate(gv_dd)
    end if

    if(allocated(gv_j)) then
       deallocate(gv_j)
    end if

    status=0

  end subroutine emission_co2_ocean_done

end module emission_co2_ocean
